@@ -1,10 +1,3 @@
-/* Extended Module Player
- * Copyright (C) 1996-1999 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See doc/COPYING
- * for more information.
- */
 
 #ifndef __PERIOD_H
 #define __PERIOD_H
@@ -23,10 +16,10 @@
 #define AMIGA_LIMIT_UPPER 108
 #define AMIGA_LIMIT_LOWER 907
 
-int	note_to_period		(int, int);
+int	note_to_period		(int, int, int);
 int	note_to_period_mix	(int, int);
 int	period_to_note		(int);
-int	period_to_bend		(int, int, int, int, int, int);
+int	period_to_bend		(int, int, int, int, int);
 void	c2spd_to_note		(int, int *, int *);
 
 #endif /* __PERIOD_H */
