@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: ftm_load.c,v 1.6 2007/11/15 22:36:54 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -23,7 +21,7 @@
 struct ftm_instrument {
 	uint8 name[30];		/* Instrument name */
 	uint8 unknown[2];
-} PACKED;
+};
 
 struct ftm_header {
 	uint8 id[4];		/* "FTMN" ID string */
@@ -33,7 +31,7 @@ struct ftm_header {
 	uint8 title[32];	/* Module title */
 	uint8 author[32];	/* Module author */
 	uint8 unknown2[2];
-} PACKED;
+};
 
 int ftm_load(FILE * f)
 {
