@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: imf_load.c,v 1.19 2007/11/29 17:43:19 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -254,7 +252,7 @@ static int imf_load(struct xmp_context *ctx, FILE *f, const int start)
 	r = 0;
 
 	while (--pat_len >= 0) {
-	    fread (&b, 1, 1, f);
+	    b = read8(f);
 
 	    if (b == IMF_EOR) {
 		r++;
@@ -339,7 +337,7 @@ static int imf_load(struct xmp_context *ctx, FILE *f, const int start)
 	str_adj ((char *) ii.name);
 	strncpy ((char *) m->xxih[i].name, ii.name, 24);
 
-	memcpy (m->xxim[i].ins, ii.map, 96);
+	memcpy (m->xxim[i].ins, ii.map, XXM_KEY_MAX);
 
 	if (V(1) && (strlen((char *) ii.name) || ii.nsm))
 	    report ("\n[%2X] %-31.31s %2d %4x %c%c%c ",
