@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr.
- *
- * $Id: it_load.c,v 1.43 2007/12/05 11:10:55 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr.
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -73,7 +71,7 @@ static uint8 fx[] = {
 	/* Q */ FX_MULTI_RETRIG,
 	/* R */ FX_TREMOLO,
 	/* S */ FX_XTND,
-	/* T */ FX_S3M_BPM,
+	/* T */ FX_IT_BPM,
 	/* U */ FX_NONE,
 	/* V */ FX_GLOBALVOL,
 	/* W */ FX_G_VOLSLIDE,
@@ -169,6 +167,12 @@ static void xlat_fx(int c, struct xxm_event *e)
 	    e->fxp *= 2;
 	}
 	break;
+    case FX_TREMOR:
+	if (!LSN(e->fxp))
+	    e->fxp |= 0x01; 
+	if (!MSN(e->fxp))
+	    e->fxp |= 0x10; 
+	break;
     case FX_NONE:		/* No effect */
 	e->fxt = e->fxp = 0;
 	break;
@@ -220,6 +224,22 @@ static void xlat_volfx(struct xxm_event *event)
 }
 
 
+static void fix_name(uint8 *s, int l)
+{
+    int i;
+
+    /* IT names can have 0 at start of data, replace with space */
+    for (l--, i = 0; i < l; i++) {
+	if (s[i] == 0)
+	    s[i] = ' ';
+    }
+    for (i--; i >= 0 && s[i] == ' '; i--) {
+	if (s[i] == ' ')
+	    s[i] = 0;
+    }
+}
+
+
 static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 {
     struct xmp_player_context *p = &ctx->p;
@@ -233,9 +253,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
     struct it_sample_header ish;
     struct it_envelope env;
     uint8 b, mask[L_CHANNELS];
-    uint16 x16;
     int max_ch, flag;
-    int inst_map[120], inst_rmap[96];
+    int inst_map[120], inst_rmap[XXM_KEY_MAX];
     char tracker_name[80];
     int mpt = 0;	/* ModPlug Tracker has quirks */
 
@@ -276,23 +295,25 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
     m->xxh->ins = ifh.insnum;
     m->xxh->smp = ifh.smpnum;
     m->xxh->pat = ifh.patnum;
-    if (m->xxh->ins)		/* Sample mode has no instruments */
-	pp_ins = calloc (4, m->xxh->ins);
-    pp_smp = calloc (4, m->xxh->smp);
-    pp_pat = calloc (4, m->xxh->pat);
+    pp_ins = m->xxh->ins ? calloc(4, m->xxh->ins) : NULL;
+    pp_smp = calloc(4, m->xxh->smp);
+    pp_pat = calloc(4, m->xxh->pat);
     m->xxh->tpo = ifh.is;
     m->xxh->bpm = ifh.it;
     m->xxh->flg = ifh.flags & IT_LINEAR_FREQ ? XXM_FLG_LINEAR : 0;
     m->xxh->flg |= (ifh.flags & IT_USE_INST) && (ifh.cmwt >= 0x200) ?
 					XXM_FLG_INSVOL : 0;
+
+    m->xxh->chn = 64;	/* Effects in muted channels are still processed! */
+
     for (i = 0; i < 64; i++) {
-	if (ifh.chpan[i] == 100)
+	if (ifh.chpan[i] == 100)	/* Surround -> center */
 	    ifh.chpan[i] = 32;
 
-	if (~ifh.chpan[i] & 0x80)
-	    m->xxh->chn = i + 1;
-	else
+	if (ifh.chpan[i] & 0x80) {	/* Channel mute */
 	    ifh.chvol[i] = 0;
+	    m->xxc[i].flg |= XXM_CHANNEL_MUTE;
+	}
 
 	if (ifh.flags & IT_STEREO) {
 	    m->xxc[i].pan = (int)ifh.chpan[i] * 0x80 >> 5;
@@ -326,6 +347,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
     m->c4rate = C4_NTSC_RATE;
     m->fetch |= XMP_CTL_FINEFX | XMP_CTL_ENVFADE;
 
+    /* Identify tracker */
+
     switch (ifh.cwt >> 8) {
     case 0x00:
 	sprintf(tracker_name, "unmo3");
@@ -341,6 +364,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    sprintf(tracker_name, "Impulse Tracker 2.14v3");
 	} else if (ifh.cwt == 0x0217) {
 	    sprintf(tracker_name, "Impulse Tracker 2.14v5");
+	} else if (ifh.cwt == 0x0214 && !memcmp(ifh.rsvd2, "CHBI", 4)) {
+	    sprintf(tracker_name, "Chibi Tracker");
 	} else {
 	    sprintf(tracker_name, "Impulse Tracker %d.%02x",
 			(ifh.cwt & 0x0f00) >> 8, ifh.cwt & 0xff);
@@ -352,6 +377,7 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	} else {
 	    sprintf(tracker_name, "unknown (%04x)", ifh.cwt);
 	}
+	break;
     case 0x10:
 	sprintf(tracker_name, "Schism Tracker %d.%02x",
 			(ifh.cwt & 0x0f00) >> 8, ifh.cwt & 0xff);
@@ -373,21 +399,26 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
     if (~ifh.flags & IT_USE_INST)
 	m->xxh->ins = m->xxh->smp;
 
-    if (V(2) && ifh.special & IT_HAS_MSG) {
-	report("\nMessage length : %d\n| ", ifh.msglen);
-	i = ftell (f);
+    if (ifh.special & IT_HAS_MSG) {
+	if ((m->comment = malloc(ifh.msglen + 1)) == NULL)
+	    return -1;
+	i = ftell(f);
 	fseek(f, start + ifh.msgofs, SEEK_SET);
+	reportv(ctx, 2, "\nMessage length : %d\n| ", ifh.msglen);
 	for (j = 0; j < ifh.msglen; j++) {
-	    fread (&b, 1, 1, f);
+	    b = read8(f);
 	    if (b == '\r')
 		b = '\n';
 	    if ((b < 32 || b > 127) && b != '\n' && b != '\t')
 		b = '.';
-	    report ("%c", b);
+	    m->comment[j] = b;
+	    reportv(ctx, 2, "%c", b);
 	    if (b == '\n')
-		report ("| ");
+		reportv(ctx, 2, "| ");
 	}
-	fseek(f, start + i, SEEK_SET);
+	m->comment[j] = 0;
+
+	fseek(f, i, SEEK_SET);
     }
 
     INSTRUMENT_INIT();
@@ -435,6 +466,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    i2h.rsvd1 = read8(f);
 	    fread(&i2h.name, 26, 1, f);
 
+	    fix_name(i2h.name, 26);
+
 	    i2h.ifc = read8(f);
 	    i2h.ifr = read8(f);
 	    i2h.mch = read8(f);
@@ -495,10 +528,10 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    }
 
 	    /* See how many different instruments we have */
-	    for (j = 0; j < 96; j++)
+	    for (j = 0; j < XXM_KEY_MAX; j++)
 		inst_map[j] = -1;
 
-	    for (k = j = 0; j < 96; j++) {
+	    for (k = j = 0; j < XXM_KEY_MAX; j++) {
 		c = i2h.keys[25 + j * 2] - 1;
 		if (c < 0) {
 		    m->xxim[i].ins[j] = 0xff;	/* No sample */
@@ -530,7 +563,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	        }
 	    }
 
-	    reportv(ctx, 1, "\n[%2X] %-26.26s %-4.4s %-4.4s %-4.4s %4d %4d  %2x "
+	    reportv(ctx, 1,
+			"\n[%2X] %-26.26s %-4.4s %-4.4s %-4.4s %4d %4d  %2x "
 			"%02x %c%c%c %3d %02x %02x ",
 		i, i2h.name,
 		i2h.nna < 4 ? nna[i2h.nna] : "none",
@@ -572,6 +606,9 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    i1h.rsvd2 = read8(f);
 
 	    fread(&i1h.name, 26, 1, f);
+
+	    fix_name(i1h.name, 26);
+
 	    fread(&i1h.rsvd3, 6, 1, f);
 	    fread(&i1h.keys, 240, 1, f);
 	    fread(&i1h.epoint, 200, 1, f);
@@ -597,10 +634,10 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    }
 	    
 	    /* See how many different instruments we have */
-	    for (j = 0; j < 96; j++)
+	    for (j = 0; j < XXM_KEY_MAX; j++)
 		inst_map[j] = -1;
 
-	    for (k = j = 0; j < 96; j++) {
+	    for (k = j = 0; j < XXM_KEY_MAX; j++) {
 		c = i1h.keys[25 + j * 2] - 1;
 		if (c < 0) {
 		    m->xxim[i].ins[j] = 0xff;	/* No sample */
@@ -666,6 +703,8 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	ish.vol = read8(f);
 	fread(&ish.name, 26, 1, f);
 
+	fix_name(ish.name, 26);
+
 	ish.convert = read8(f);
 	ish.dfp = read8(f);
 	ish.length = read32l(f);
@@ -681,8 +720,11 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	ish.vir = read8(f);
 	ish.vit = read8(f);
 
+	/* Changed to continue to allow use-brdg.it and use-funk.it to
+	 * load correctly (both IT 2.04)
+	 */
 	if (ish.magic != MAGIC_IMPS)
-	    return -2;
+	    continue;
 	
 	if (ish.flags & IT_SMP_16BIT) {
 	    m->xxs[i].len = ish.length * 2;
@@ -704,8 +746,9 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    m->xxi[i][0].pan = 0x80;
 	    m->xxi[i][0].sid = i;
 	    m->xxih[i].nsm = !!(m->xxs[i].len);
-
 	    copy_adjust(m->xxih[i].name, ish.name, 24);
+	} else {
+	    copy_adjust(m->xxs[i].name, ish.name, 24);
 	}
 
 	if (V(2) || (~ifh.flags & IT_USE_INST && V(1))) {
@@ -814,11 +857,11 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	m->xxp[i]->rows = read16l(f);
 	TRACK_ALLOC (i);
 	memset (mask, 0, L_CHANNELS);
-	fread (&x16, 2, 1, f);
-	fread (&x16, 2, 1, f);
+	read16l(f);
+	read16l(f);
 
 	while (--pat_len >= 0) {
-	    fread (&b, 1, 1, f);
+	    b = read8(f);
 	    if (!b) {
 		r++;
 		continue;
@@ -826,7 +869,7 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	    c = (b - 1) & 63;
 
 	    if (b & 0x80) {
-		fread (&mask[c], 1, 1, f);
+		mask[c] = read8(f);
 		pat_len--;
 	    }
 	    /*
@@ -837,7 +880,7 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	     */
 	    event = c >= m->xxh->chn ? &dummy : &EVENT (i, c, r);
 	    if (mask[c] & 0x01) {
-		fread (&b, 1, 1, f);
+		b = read8(f);
 
 		if (b > 0x7f && b < 0xfd)
 			b = 0;
@@ -862,21 +905,20 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 		pat_len--;
 	    }
 	    if (mask[c] & 0x02) {
-		fread (&b, 1, 1, f);
+		b = read8(f);
 		lastevent[c].ins = event->ins = b;
 		pat_len--;
 	    }
 	    if (mask[c] & 0x04) {
-		fread (&b, 1, 1, f);
+		b = read8(f);
 		lastevent[c].vol = event->vol = b;
 		xlat_volfx (event);
 		pat_len--;
 	    }
 	    if (mask[c] & 0x08) {
-		fread (&b, 1, 1, f);
+		b = read8(f);
 		event->fxt = b;
-		fread (&b, 1, 1, f);
-		event->fxp = b;
+		event->fxp = read8(f);
 		xlat_fx (c, event);
 		lastevent[c].fxt = event->fxt;
 		lastevent[c].fxp = event->fxp;
@@ -899,7 +941,7 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	}
 	reportv(ctx, 0, ".");
 
-	/* Sweep channels, look for unused tracks */
+	/* Scan channels, look for unused tracks */
 	for (c = m->xxh->chn - 1; c >= max_ch; c--) {
 	    for (flag = j = 0; j < m->xxt[m->xxp[i]->info[c].index]->rows; j++) {
 		event = &EVENT (i, c, j);
@@ -914,11 +956,19 @@ static int it_load(struct xmp_context *ctx, FILE *f, const int start)
 	}
     }
 
+    free(pp_pat);
+    free(pp_smp);
+    if (pp_ins)		/* sample mode has no instruments */
+	free(pp_ins);
+
     m->xxh->chn = max_ch + 1;
-    m->fetch |= ifh.flags & IT_USE_INST ? XMP_MODE_IT : XMP_MODE_ST3;
-    m->fetch |= XMP_CTL_INSPRI | XMP_CTL_VIRTUAL | XMP_CTL_FILTER;
+    /*m->fetch |= ifh.flags & IT_USE_INST ? XMP_MODE_IT : XMP_MODE_ST3;*/
+    m->fetch |= XMP_MODE_IT;
+    m->fetch |= XMP_CTL_VIRTUAL | XMP_CTL_FILTER;
+    if (~ifh.flags & IT_LINK_GXX)
+	m->fetch |= XMP_CTL_UNISLD;
 
     reportv(ctx, 0, "\n");
 
-    return XMP_OK;
+    return 0;
 }
