@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: mdl_load.c,v 1.29 2007/11/12 19:30:32 cmatsuoka Exp $
  */
 
 /* Note: envelope switching (effect 9) and sample status change (effect 8)
@@ -175,7 +173,7 @@ static void xlat_fx2(uint8 *t, uint8 *p)
 }
 
 
-static unsigned int get_bits(char i, uint8 **buf, int* len)
+static unsigned int get_bits(char i, uint8 **buf, int *len)
 {
     static uint32 b = 0, n = 32;
     unsigned int x;
@@ -187,9 +185,11 @@ static unsigned int get_bits(char i, uint8 **buf, int* len)
 	return 0;
     }
 
-    x = b & ((1 << i) - 1);
+    x = b & ((1 << i) - 1);	/* get i bits */
     b >>= i;
     if ((n -= i) <= 24) {
+	if (*len == 0)		/* FIXME: last few bits can't be consumed */
+		return x;
 	b |= (uint32)(*(*buf)++) << n;
 	n += 8; (*len)--;
     }
@@ -328,7 +328,7 @@ static void get_chunk_pa(struct xmp_context *ctx, int size, FILE *f)
     int x;
 
     m->xxh->pat = read8(f);
-    m->xxh->trk = m->xxh->pat * m->xxh->chn;	/* Max */
+    m->xxh->trk = m->xxh->pat * m->xxh->chn + 1;	/* Max */
 
     PATTERN_INIT();
     reportv(ctx, 0, "Stored patterns: %d ", m->xxh->pat);
@@ -357,7 +357,7 @@ static void get_chunk_p0(struct xmp_context *ctx, int size, FILE *f)
     uint16 x16;
 
     m->xxh->pat = read8(f);
-    m->xxh->trk = m->xxh->pat * m->xxh->chn;	/* Max */
+    m->xxh->trk = m->xxh->pat * m->xxh->chn + 1;	/* Max */
 
     PATTERN_INIT();
     reportv(ctx, 0, "Stored patterns: %d ", m->xxh->pat);
@@ -384,6 +384,7 @@ static void get_chunk_tr(struct xmp_context *ctx, int size, FILE *f)
     struct xxm_track *track;
 
     m->xxh->trk = read16l(f) + 1;
+    m->xxt = realloc(m->xxt, sizeof (struct xxm_track *) * m->xxh->trk);
 
     reportv(ctx, 0, "Stored tracks  : %d ", m->xxh->trk);
 
@@ -476,7 +477,6 @@ static void get_chunk_ii(struct xmp_context *ctx, int size, FILE *f)
     int map, last_map;
 
     m->xxh->ins = read8(f);
-
     reportv(ctx, 0, "Instruments    : %d ", m->xxh->ins);
 
     INSTRUMENT_INIT();
@@ -494,6 +494,9 @@ static void get_chunk_ii(struct xmp_context *ctx, int size, FILE *f)
 
 	m->xxi[i] = calloc (sizeof (struct xxm_instrument), m->xxih[i].nsm);
 
+	for (j = 0; j < XXM_KEY_MAX; j++)
+	    m->xxim[i].ins[j] = -1;
+
 	for (last_map = j = 0; j < m->xxih[i].nsm; j++) {
 	    int x;
 
@@ -501,7 +504,7 @@ static void get_chunk_ii(struct xmp_context *ctx, int size, FILE *f)
 	    map = read8(f);
 	    m->xxi[i][j].vol = read8(f);
 	    for (k = last_map; k <= map; k++) {
-		if (k < 96)
+		if (k < XXM_KEY_MAX)
 		    m->xxim[i].ins[k] = j;
 	    }
 	    last_map = map + 1;
@@ -637,7 +640,6 @@ static void get_chunk_i0(struct xmp_context *ctx, int size, FILE *f)
 
     INSTRUMENT_INIT();
 
-    m->xxs = calloc (sizeof (struct xxm_sample), m->xxh->smp);
     packinfo = calloc (sizeof (int), m->xxh->smp);
 
     for (i = 0; i < m->xxh->ins; i++) {
@@ -738,6 +740,8 @@ static void get_chunk_sa(struct xmp_context *ctx, int size, FILE *f)
 	reportv(ctx, 0, ".");
     }
     reportv(ctx, 0, "\n");
+
+    free(packinfo);
 }
 
 static void get_chunk_ve(struct xmp_context *ctx, int size, FILE *f)
@@ -858,16 +862,22 @@ static int mdl_load(struct xmp_context *ctx, FILE *f, const int start)
 
     iff_release();
 
+#define EVENT(a, c, r)  m->xxt[m->xxp[a]->info[c].index]->event[r]
+
     /* Re-index instruments & samples */
 
     for (i = 0; i < m->xxh->pat; i++)
 	for (j = 0; j < m->xxp[i]->rows; j++)
 	    for (k = 0; k < m->xxh->chn; k++)
-		for (l = 0; l < m->xxh->ins; l++)
+		for (l = 0; l < m->xxh->ins; l++) {
+		    if (j >= m->xxt[m->xxp[i]->info[k].index]->rows)
+			continue;
+		    
 		    if (EVENT(i, k, j).ins && EVENT(i, k, j).ins == i_index[l]) {
 		    	EVENT(i, k, j).ins = l + 1;
 			break;
 		    }
+		}
 
     for (i = 0; i < m->xxh->ins; i++) {
 
