@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: mmd1_load.c,v 1.36 2007/12/04 22:22:06 cmatsuoka Exp $
  */
 
 /*
@@ -71,7 +69,7 @@ static char *inst_type[] = {
 	"EXT",			/*  7 */
 };
 
-static int bpmon, bpmlen;
+static int bpm_on, bpmlen;
 
 static void xlat_fx(struct xxm_event *event)
 {
@@ -105,9 +103,10 @@ static void xlat_fx(struct xxm_event *event)
 			break;
 		} else if (event->fxp <= 0xf0) {
 			event->fxt = FX_S3M_BPM;
+			if (bpm_on)
+				event->fxp = event->fxp * 8 / bpmlen;
 			break;
-		}
-		switch (event->fxp) {
+		} else switch (event->fxp) {
 		case 0xf1:	/* Play note twice */
 			event->fxt = FX_EXTENDED;
 			event->fxp = (EX_RETRIG << 4) | 3;
@@ -226,9 +225,9 @@ static int mmd1_load(struct xmp_context *ctx, FILE *f, const int start)
 	 */
 	m->c4rate = C4_NTSC_RATE;
 	m->fetch |= song.flags & FLAG_STSLIDE ? 0 : XMP_CTL_VSALL;
-	bpmon = song.flags2 & FLAG2_BPM;
+	bpm_on = song.flags2 & FLAG2_BPM;
 	bpmlen = 1 + (song.flags2 & FLAG2_BMASK);
-	m->fetch |= bpmon ? 0 : XMP_CTL_MEDBPM;
+	m->fetch |= bpm_on ? 0 : XMP_CTL_MEDBPM;
 
 	/* From the OctaMEDv4 documentation:
 	 *
@@ -241,14 +240,21 @@ static int mmd1_load(struct xmp_context *ctx, FILE *f, const int start)
 	/* Just a half-assed implementation of the spec above for tempo 1
 	 * in PrivInv.med
 	 */
-	if (!bpmon && song.deftempo < 10)
+	if (!bpm_on && song.deftempo < 10)
 		song.deftempo = 0x35 - song.deftempo * 2;
 
 	/* FIXME: med tempos are incorrectly handled */
-	m->xxh->tpo = song.tempo2;
-	m->xxh->bpm = bpmon ? song.deftempo * bpmlen / 4 : song.deftempo;
-	if (!bpmon && m->xxh->bpm <= 10)
-		m->xxh->bpm = m->xxh->bpm * 33 / 6;
+	if (bpm_on) {
+		m->xxh->tpo = 3;
+		m->xxh->bpm = song.deftempo * 8 / bpmlen;
+	} else {
+		m->xxh->tpo = song.tempo2;
+		m->xxh->bpm = song.deftempo;
+
+		if (m->xxh->bpm <= 10)
+			m->xxh->bpm = m->xxh->bpm * 33 / 6;
+	}
+
 	m->xxh->pat = song.numblocks;
 	m->xxh->ins = song.numsamples;
 	//m->xxh->smp = m->xxh->ins;
@@ -354,8 +360,8 @@ static int mmd1_load(struct xmp_context *ctx, FILE *f, const int start)
 	MODULE_INFO();
 
 	if (V(0)) {
-		report("BPM mode       : %s", bpmon ? "on" : "off");
-		if (bpmon)
+		report("BPM mode       : %s", bpm_on ? "on" : "off");
+		if (bpm_on)
 			report(" (length = %d)", bpmlen);
 		report("\n");
 		if (V(1) && song.playtransp)
@@ -419,11 +425,12 @@ static int mmd1_load(struct xmp_context *ctx, FILE *f, const int start)
 					e[2] = read8(f);
 
 					event = &EVENT(i, k, j);
-					if ((event->note = e[0] & 0x3f))
+					event->note = e[0] & 0x3f;
+					if (event->note)
 						event->note += 36;
 					event->ins =
-					    (e[1] >> 4) | ((e[0] & 0x80) >> 3) |
-					    ((e[0] & 0x40) >> 1);
+					    (e[1] >> 4) | ((e[0] & 0x80) >> 3)
+					    | ((e[0] & 0x40) >> 1);
 					event->fxt = e[1] & 0x0f;
 					event->fxp = e[2];
 					xlat_fx(event);
@@ -650,6 +657,31 @@ static int mmd1_load(struct xmp_context *ctx, FILE *f, const int start)
 
 	reportv(ctx, 0, "\n");
 
+	/*
+	 * Adjust event note data in patterns
+	 */
+
+	/* Restrict non-synth instruments to 3 octave range.
+	 * Checked in MMD0 with med.egypian/med.medieval from Lemmings 2
+	 * and MED.ParasolStars, MMD1 with med.Lemmings2
+	 */
+	for (i = 0; i < m->xxh->pat; i++) {
+		for (j = 0; j < m->xxp[i]->rows; j++) {
+			for (k = 0; k < m->xxh->chn; k++) {
+				event = &EVENT(i, k, j);
+
+				if (!event->note || !event->ins)
+					continue;
+
+				/* Not a synth instrument */
+				if (!p->m.med_wav_table[event->ins - 1]) {
+					while (event->note > (35 + 36))
+						event->note -= 12;
+				}
+			}
+		}
+	}
+
 	for (i = 0; i < m->xxh->chn; i++) {
 		m->xxc[i].vol = song.trkvol[i];
 		m->xxc[i].pan = (((i + 1) / 2) % 2) * 0xff;
