@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: mmd3_load.c,v 1.33 2007/11/15 22:36:54 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,7 +65,7 @@ static char *inst_type[] = {
 	"EXT",			/*  7 */
 };
 
-static int bpmon, bpmlen;
+static int bpm_on, bpmlen;
 
 static void xlat_fx(struct xxm_event *event)
 {
@@ -101,9 +99,10 @@ static void xlat_fx(struct xxm_event *event)
 			break;
 		} else if (event->fxp <= 0xf0) {
 			event->fxt = FX_S3M_BPM;
+			if (bpm_on)
+				event->fxp = event->fxp * 8 / bpmlen;
 			break;
-		}
-		switch (event->fxp) {
+		} else switch (event->fxp) {
 		case 0xf1:	/* Play note twice */
 			event->fxt = FX_EXTENDED;
 			event->fxp = (EX_RETRIG << 4) | 3;
@@ -253,9 +252,9 @@ static int mmd3_load(struct xmp_context *ctx, FILE *f, const int start)
 	 */
 	m->c4rate = C4_NTSC_RATE;
 	m->fetch |= song.flags & FLAG_STSLIDE ? 0 : XMP_CTL_VSALL;
-	bpmon = song.flags2 & FLAG2_BPM;
+	bpm_on = song.flags2 & FLAG2_BPM;
 	bpmlen = 1 + (song.flags2 & FLAG2_BMASK);
-	m->fetch |= bpmon ? 0 : XMP_CTL_MEDBPM;
+	m->fetch |= bpm_on ? 0 : XMP_CTL_MEDBPM;
 
 	/* From the OctaMEDv4 documentation:
 	 *
@@ -268,14 +267,21 @@ static int mmd3_load(struct xmp_context *ctx, FILE *f, const int start)
 	/* Just a half-assed implementation of the spec above for tempo 1
 	 * in PrivInv.med
 	 */
-	if (!bpmon && song.deftempo < 10)
+	if (!bpm_on && song.deftempo < 10)
 		song.deftempo = 0x35 - song.deftempo * 2;
 
 	/* FIXME: med tempos are incorrectly handled */
-	m->xxh->tpo = song.tempo2;
-	m->xxh->bpm = bpmon ? song.deftempo * bpmlen / 4 : song.deftempo;
-	if (!bpmon && m->xxh->bpm <= 10)
-		m->xxh->bpm = m->xxh->bpm * 33 / 6;
+	if (bpm_on) {
+		m->xxh->tpo = 3;
+		m->xxh->bpm = song.deftempo * 8 / bpmlen;
+	} else {
+		m->xxh->tpo = song.tempo2;
+		m->xxh->bpm = song.deftempo;
+
+		if (m->xxh->bpm <= 10)
+			m->xxh->bpm = m->xxh->bpm * 33 / 6;
+	}
+
 	m->xxh->pat = song.numblocks;
 	m->xxh->ins = song.numsamples;
 	//m->xxh->smp = m->xxh->ins;
@@ -376,8 +382,8 @@ static int mmd3_load(struct xmp_context *ctx, FILE *f, const int start)
 	MODULE_INFO();
 
 	if (V(0)) {
-		report("BPM mode       : %s", bpmon ? "on" : "off");
-		if (bpmon)
+		report("BPM mode       : %s", bpm_on ? "on" : "off");
+		if (bpm_on)
 			report(" (length = %d)", bpmlen);
 		report("\n");
 		if (V(1) && song.playtransp)
