@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: mod_load.c,v 1.49 2007/11/30 02:11:27 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -29,7 +27,8 @@
 
 #include <ctype.h>
 #include <sys/types.h>
-
+#include <sys/stat.h>
+#include <unistd.h>
 #include "load.h"
 #include "mod.h"
 
@@ -52,7 +51,6 @@ struct {
     { "FA04", 1, 0, "Digital Tracker", 4 },	/* Atari Falcon */
     { "FA06", 1, 0, "Digital Tracker", 6 },	/* Atari Falcon */
     { "FA08", 1, 0, "Digital Tracker", 8 },	/* Atari Falcon */
-    { "PWIZ", 1, 1, "", 4 },
     { "", 0 }
 };
 
@@ -71,6 +69,8 @@ static int mod_test(FILE *f, char *t, const int start)
 {
     int i;
     char buf[4];
+    struct stat st;
+    int smp_size, num_pat;
 
     fseek(f, start + 1080, SEEK_SET);
     fread(buf, 4, 1, f);
@@ -93,6 +93,42 @@ static int mod_test(FILE *f, char *t, const int start)
     if (mod_magic[i].ch == 0)
 	return -1;
 
+    /* Test for UNIC tracker modules
+     *
+     * From Gryzor's Pro-Wizard PW_FORMATS-Engl.guide:
+     * ``The UNIC format is very similar to Protracker... At least in the
+     * heading... same length : 1084 bytes. Even the "M.K." is present,
+     * sometimes !! Maybe to disturb the rippers.. hehe but Pro-Wizard
+     * doesn't test this only!''
+     */
+
+    /* get file size */
+    fstat(fileno(f), &st);
+    smp_size = 0;
+    fseek(f, start + 20, SEEK_SET);
+
+    /* get samples size */
+    for (i = 0; i < 31; i++) {
+	fseek(f, 22, SEEK_CUR);
+	smp_size += 2 * read16b(f);		/* Length in 16-bit words */
+	fseek(f, 6, SEEK_CUR);
+    } 
+
+    /* get number of patterns */
+    num_pat = 0;
+    fseek(f, start + 952, SEEK_SET);
+    for (i = 0; i < 128; i++) {
+	uint8 x = read8(f);
+	if (x > 0x7f)
+		break;
+	if (x > num_pat)
+	    num_pat = x;
+    }
+    num_pat++;
+
+    if (start + 1084 + num_pat * 0x300 + smp_size == st.st_size)
+	return -1;
+
     fseek(f, start + 0, SEEK_SET);
     read_title(f, t, 20);
 
@@ -149,6 +185,8 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
 	mh.ins[i].volume = read8(f);		/* Linear playback volume */
 	mh.ins[i].loop_start = read16b(f);	/* Loop start in 16-bit words */
 	mh.ins[i].loop_size = read16b(f);	/* Loop size in 16-bit words */
+
+	smp_size += 2 * mh.ins[i].size;
     }
     mh.len = read8(f);
     mh.restart = read8(f);
@@ -166,15 +204,7 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
 	}
     }
 
-    /* Prowizard hack */
-    if (memcmp(magic, "PWIZ", 4) == 0) {
-	int pos = ftell(f);
-	fseek(f, -40, SEEK_END);
-	fread(magic, 8, 1, f);
-	fread(idbuffer, 1, 32, f);
-	fseek(f, start + pos, SEEK_SET);
-	tracker = idbuffer;
-    } else if (!m->xxh->chn) {
+    if (!m->xxh->chn) {
 	if (!strncmp(magic + 2, "CH", 2) &&
 	    isdigit(magic[0]) && isdigit(magic[1])) {
 	    if ((m->xxh->chn = (*magic - '0') *
@@ -214,8 +244,6 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
     INSTRUMENT_INIT();
 
     for (i = 0; i < m->xxh->ins; i++) {
-	smp_size += 2 * mh.ins[i].size;
-
 	m->xxi[i] = calloc (sizeof (struct xxm_instrument), 1);
 	m->xxs[i].len = 2 * mh.ins[i].size;
 	m->xxs[i].lps = lps_mult * mh.ins[i].loop_start;
@@ -279,20 +307,6 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
 	goto skip_test;
     }
 
-    /* Test for UNIC tracker modules
-     *
-     * From Gryzor's Pro-Wizard PW_FORMATS-Engl.guide:
-     * ``The UNIC format is very similar to Protracker... At least in the
-     * heading... same length : 1084 bytes. Even the "M.K." is present,
-     * sometimes !! Maybe to disturb the rippers.. hehe but Pro-Wizard
-     * doesn't test this only!''
-     */
-
-    else if (sizeof (struct mod_header) + m->xxh->pat * 0x300 +
-	smp_size == m->size) {
-	return -1;
-    }
-
     /* Test for Protracker song files
      */
     else if ((ptsong = (!strncmp((char *)magic, "M.K.", 4) &&
@@ -322,6 +336,7 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
     if (m->xxh->chn != 4 && mh.restart == 0x7f) {
 	tracker = "Scream Tracker 3?";
 	ptkloop = 0;
+	m->xxh->flg &= ~XXM_FLG_MODRNG;
     }
 
     if (m->xxh->chn == 4 && mh.restart == 0x7f) {
@@ -379,6 +394,7 @@ static int mod_load(struct xmp_context *ctx, FILE *f, const int start)
 		} else if (m->xxh->chn == 6 || m->xxh->chn == 8) {
 	    	    tracker = "FastTracker 1.01?";
 		    ptkloop = 0;
+		    m->xxh->flg &= ~XXM_FLG_MODRNG;
 		} else {
 	    	    tracker = "unknown tracker";
 		    ptkloop = 0;
@@ -503,8 +519,10 @@ skip_test:
     }
     reportv(ctx, 0, "\n");
 
-    if (m->xxh->chn > 4)
+    if (m->xxh->chn > 4) {
+	m->xxh->flg &= ~XXM_FLG_MODRNG;
 	m->fetch |= XMP_MODE_FT2;
+    }
 
     return 0;
 }
