@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: pt3_load.c,v 1.20 2007/11/30 02:11:27 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -156,6 +154,8 @@ static int ptdt_load(struct xmp_context *ctx, FILE *f, const int start)
 	fread(&mh.order, 128, 1, f);
 	fread(&mh.magic, 4, 1, f);
 
+	m->xxh->ins = 31;
+	m->xxh->smp = m->xxh->ins;
 	m->xxh->chn = 4;
 	m->xxh->len = mh.len;
 	m->xxh->rst = mh.restart;
