@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: s3m_load.c,v 1.33 2007/12/05 11:10:55 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -326,6 +324,10 @@ static int s3m_load(struct xmp_context *ctx, FILE *f, const int start)
 			(sfh.version & 0x0f00) >> 8, sfh.version & 0xff);
 	}
 	break;
+    case 4:
+	sprintf(tracker_name, "Schism Tracker %d.%02x",
+		(sfh.version & 0x0f00) >> 8, sfh.version & 0xff);
+	break;
     default:
 	snprintf(tracker_name, 80, "unknown (%04x)", sfh.version);
     }
@@ -514,6 +516,9 @@ static int s3m_load(struct xmp_context *ctx, FILE *f, const int start)
     }
     reportv(ctx, 0, "\n");
 
+    free(pp_pat);
+    free(pp_ins);
+
     m->fetch |= XMP_MODE_ST3;
 
     return 0;
