@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: main.c,v 1.39 2007/12/05 23:39:27 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -31,7 +29,9 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
+#endif
 #include <unistd.h>
 
 #ifdef __AROS__
@@ -59,6 +59,7 @@
 #endif
 
 #ifdef WIN32
+#include <windows.h>
 #include "conio.h"
 #endif
 
@@ -90,14 +91,16 @@ static int max_nch = 0;
 static double rows, tot_nch;
 int skip = 0;
 
+#ifdef HAVE_SIGNAL_H
 static int sigusr = 0;
+#endif
 
 void get_options (int, char **, struct xmp_options *, xmp_context);
 
 static xmp_context ctx;
 
 
-static int set_tty ()
+static int set_tty()
 {
 #ifdef HAVE_TERMIOS_H
     struct termios t;
@@ -120,7 +123,7 @@ static int set_tty ()
 }
 
 
-static int reset_tty ()
+static int reset_tty()
 {
 #ifdef HAVE_TERMIOS_H
     if (background)
@@ -135,6 +138,9 @@ static int reset_tty ()
     return 0;
 }
 
+
+#ifdef HAVE_SIGNAL_H
+
 #ifdef SIGTSTP
 static void sigtstp_handler()
 {
@@ -180,7 +186,7 @@ static void sigusr_handler (int i)
 }
 
 
-static void cleanup (int s)
+static void cleanup(int s)
 {
     signal(SIGTERM, SIG_DFL);
     signal(SIGINT, SIG_DFL);
@@ -190,7 +196,7 @@ static void cleanup (int s)
     signal(SIGFPE, SIG_DFL);
     signal(SIGSEGV, SIG_DFL);
 
-    fprintf (stderr, "\n*** Interrupted: signal %d caught\n", s);
+    fprintf(stderr, "\n*** Interrupted: signal %d caught\n", s);
     xmp_stop_module(ctx);
     xmp_close_audio(ctx);
 
@@ -199,6 +205,8 @@ static void cleanup (int s)
     exit (-2);
 }
 
+#endif /* HAVE_SIGNAL_H */
+
 
 static void process_echoback(unsigned long i)
 {
@@ -363,8 +371,10 @@ int main (int argc, char **argv)
 {
     int i, t, lf_flag, first, num_mod, verb = 0;
     time_t t0, t1;
+#ifndef WIN32
     struct timeval tv;
     struct timezone tz;
+#endif
     struct xmp_options *opt;
 #ifdef HAVE_SYS_RTPRIO_H
     struct rtprio rtp;
@@ -389,8 +399,12 @@ int main (int argc, char **argv)
 	exit (-1);
     }
 
+#ifndef WIN32
     gettimeofday(&tv, &tz);
     srand(tv.tv_usec);
+#else
+    srand(GetTickCount());
+#endif
 
     if (randomize)
 	shuffle(argc - optind + 1, &argv[optind - 1]);
@@ -434,7 +448,7 @@ int main (int argc, char **argv)
 
     if (opt->verbosity) {
 	fprintf (stderr, "Extended Module Player %s %s\n"
-	"Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr\n",
+	"Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr\n",
 	    xmp_version, xmp_date);
 #ifdef __EMX__
 	fprintf(stderr, "OS/2 Port by Kevin Langman (langman@earthling.net)\n" );
@@ -476,6 +490,7 @@ int main (int argc, char **argv)
    }
 #endif
 
+#ifdef HAVE_SIGNAL_H
     signal(SIGTERM, cleanup);
     signal(SIGINT, cleanup);
     signal(SIGFPE, cleanup);
@@ -492,13 +507,15 @@ int main (int argc, char **argv)
 #ifdef SIGUSR1
     signal(SIGUSR1, sigusr_handler);
     signal(SIGUSR2, sigusr_handler);
+#endif
 #endif
 
     set_tty ();
 
     time (&t0);
 
-    for (first = optind, num_mod = lf_flag = 0; optind < argc; optind++) {
+    num_mod = lf_flag = 0;
+    for (first = optind; optind < argc; optind++) {
 	if (getprevious) {
 	    optind -= 2;
 	    if (optind < first) {
