@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: options.c,v 1.31 2007/11/18 12:47:19 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,6 +27,7 @@ extern int probeonly;
 extern int loadonly;
 extern int randomize;
 extern int nocmd;
+extern int checksum;
 #ifdef HAVE_SYS_RTPRIO_H
 extern int rt;
 #endif
@@ -47,6 +46,7 @@ extern int rt;
 #define OPT_FIXLOOP	0x10d
 #define OPT_CRUNCH	0x10e
 #define OPT_NOFILTER	0x10f
+#define OPT_VBLANK	0x110
 
 
 static void exclude_formats(char *list)
@@ -102,7 +102,7 @@ static void copyright_header()
 {
     printf("Extended Module Player %s %s\n", xmp_version, xmp_date);
     printf(
-"Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr\n"
+"Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr\n"
 "Portions Copyright (C) 1996-1997 Takashi Iwai, (C) 1988 Tammo Hinrichs,\n"
 "(C) 1989 Rich Gopstein and Harris Corporation, (C) 1997 Bert Jahn, (C) 1998\n"
 "Sylvain Chipaux, (C) 1998,2000 Olivier Lapicque, (C) 1999 Tatsuyuki Satoh,\n"
@@ -118,7 +118,8 @@ static void usage(char *s, struct xmp_options *opt)
     int i;
 
     copyright_header ();
-    printf ("%s\n", xmp_build); 
+    if (xmp_build)
+	printf ("%s\n", xmp_build); 
 
     printf ("Usage: %s [options] [modules]\n", s);
 
@@ -172,6 +173,7 @@ static void usage(char *s, struct xmp_options *opt)
 "   -s --start num         Start from the specified order\n"
 "   -T --tempo num         Initial tempo (default 6)\n"
 "   -t --time num          Maximum playing time in seconds\n"
+"   --vblank               Force vblank timing in Amiga modules (no CIA)\n"
 
 "\nPlayer sound options:\n"
 "   -8 --8bit              Convert 16 bit samples to 8 bit\n"
@@ -185,8 +187,10 @@ static void usage(char *s, struct xmp_options *opt)
 "   --stereo               Stereo output\n"
 
 "\nSoftware mixer options:\n"
+"   -a --amplify {0|1|2|3} Amplification factor: 0=Normal, 1=x2, 2=x4, 3=x8\n"
 "   -b --bits {8|16}       Software mixer resolution (8 or 16 bits)\n"
 "   -c --stdout            Mix the module to stdout\n"
+"   -F --click-filter      Apply LPF filter to remove clicks\n"
 "   -f --frequency rate    Sampling rate in hertz (default %d Hz)\n"
 "   -i --interpolate       Use linear interpolation (default %s)\n"
 "   -n --nearest           Use nearest neighbor interpolation\n"
@@ -200,7 +204,7 @@ static void usage(char *s, struct xmp_options *opt)
 "   -q --quiet             Quiet mode (verbosity level = 0)\n"
 "   -V --version           Print version information\n"
 "   -v --verbose           Verbose mode (incremental)\n"
-	,opt->mix,
+	, opt->mix,
 	opt->freq,
 	opt->flags & XMP_CTL_ITPT ? "enabled" : "disabled"
     );
@@ -210,15 +214,17 @@ static void usage(char *s, struct xmp_options *opt)
 void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx)
 {
     int optidx = 0;
-#define OPTIONS "8b:cD:d:f:hilM:mno:P:qRrS:s:T:t:uVvx:"
+#define OPTIONS "8a:b:cD:d:Ff:hilM:mno:P:qRrS:s:T:t:uVvx:"
     static struct option lopt[] = {
 	{ "8bit",		 0, 0, '8' },
+	{ "amplify",		 1, 0, 'a' },
 	{ "bits",		 1, 0, 'b' },
 	{ "chorus",		 1, 0, OPT_CHORUS },
 	{ "crunch",		 1, 0, OPT_CRUNCH },
 	{ "driver",		 1, 0, 'd' },
 	{ "fix-sample-loops",	 0, 0, OPT_FIXLOOP },
 	{ "frequency",		 1, 0, 'f' },
+	{ "click-filter",	 0, 0, 'F' },
 	{ "offset-bug-emulation",0, 0, OPT_FX9BUG },
 	{ "help",		 0, 0, 'h' },
 	{ "interpolate",	 0, 0, 'i' },
@@ -248,6 +254,7 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	{ "tempo",		 1, 0, 'T' },
 	{ "time",		 1, 0, 't' },
 	{ "unsigned",		 0, 0, 'u' },
+	{ "vblank",		 0, 0, OPT_VBLANK },
 	{ "version",		 0, 0, 'V' },
 	{ "verbose",		 0, 0, 'v' },
 	{ "exclude-formats",	 1, 0, 'x' },
@@ -260,6 +267,9 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	case '8':
 	    opt->flags |= XMP_CTL_8BIT;
 	    break;
+	case 'a':
+	    opt->amplify = atoi(optarg);
+	    break;
 	case 'b':
 	    opt->resol = atoi(optarg);
 	    if (opt->resol != 8 && opt->resol != 16)
@@ -286,6 +296,10 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	case OPT_FX9BUG:
 	    opt->flags |= XMP_CTL_FX9BUG;
 	    break;
+	case 'F':
+	    opt->flags |= XMP_CTL_FILTER;
+	    opt->cf_cutoff = 0xfd;
+	    break;
 	case 'f':
 	    opt->freq = strtoul(optarg, NULL, 0);
 	    break;
@@ -317,6 +331,10 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	    break;
 	case 'o':
 	    opt->outfile = optarg;
+	    if (strlen(optarg) >= 4 &&
+			!strcasecmp(optarg + strlen(optarg) - 4, ".wav")) {
+		opt->drv_id = "wav";
+	    }
 	    break;
 	case 'P':
 	    opt->mix = strtoul(optarg, NULL, 0);
@@ -385,6 +403,9 @@ void get_options(int argc, char **argv, struct xmp_options *opt, xmp_context ctx
 	case 'u':
 	    opt->outfmt |= XMP_FMT_UNS;
 	    break;
+	case OPT_VBLANK:
+	    opt->flags |= XMP_CTL_VBLANK;
+	    break;
 	case 'V':
 	    printf ("Extended Module Player %s\n", xmp_version);
 	    exit (0);
