@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: control.c,v 1.35 2007/11/20 23:38:19 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -46,10 +44,12 @@ void *xmp_create_context()
 	o->big_endian = (*(char *)&w == 0x00);
 
 	/* Set defaults */
+	o->amplify = DEFAULT_AMPLIFY;
 	o->freq = 44100;
-	o->mix = 80;
+	o->mix = 70;
 	o->resol = 16;
 	o->flags = XMP_CTL_DYNPAN | XMP_CTL_FILTER | XMP_CTL_ITPT;
+	o->cf_cutoff = 0;
 
 	/* Set max number of voices per channel */
 	o->maxvoc = 16;
@@ -96,9 +96,9 @@ void xmp_init(xmp_context ctx, int argc, char **argv)
 			break;
 	}
 	if (num >= argc)
-		xmpi_read_rc((struct xmp_context *)ctx);
+		_xmp_read_rc((struct xmp_context *)ctx);
 
-	xmpi_tell_wait();
+	/* _xmp_tell_wait(); */
 }
 
 inline int xmp_open_audio(xmp_context ctx)
@@ -177,7 +177,7 @@ int xmp_player_ctl(xmp_context ctx, int cmd, int arg)
 		break;
 	}
 
-	return XMP_OK;
+	return 0;
 }
 
 int xmp_play_module(xmp_context ctx)
@@ -187,7 +187,7 @@ int xmp_play_module(xmp_context ctx)
 	int t;
 
 	time(&t0);
-	xmpi_player_start((struct xmp_context *)ctx);
+	_xmp_player_start((struct xmp_context *)ctx);
 	time(&t1);
 	t = difftime(t1, t0);
 
@@ -196,55 +196,27 @@ int xmp_play_module(xmp_context ctx)
 	return t;
 }
 
-void xmp_release_module(xmp_context ctx)
+#ifdef TEST_OPEN_LOOP
+
+int xmp_play_module2(xmp_context ctx)
 {
-	struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
-	struct xmp_mod_context *m = &p->m;
-	int i;
+	struct xmp_options *o = &((struct xmp_context *)ctx)->o;
+	time_t t0, t1;
+	int t;
 
-	_D(_D_INFO "Freeing memory");
+	time(&t0);
+	xmp_player_start((struct xmp_context *)ctx);
+	while (xmp_player_loop((struct xmp_context *)ctx) == 0);
+	xmp_player_end((struct xmp_context *)ctx);
+	time(&t1);
+	t = difftime(t1, t0);
 
-	if (m->med_vol_table) {
-		for (i = 0; i < m->xxh->ins; i++)
-			if (m->med_vol_table[i])
-				free(m->med_vol_table[i]);
-		free(m->med_vol_table);
-	}
+	o->start = 0;
 
-	if (m->med_wav_table) {
-		for (i = 0; i < m->xxh->ins; i++)
-			if (m->med_wav_table[i])
-				free(m->med_wav_table[i]);
-		free(m->med_wav_table);
-	}
-
-	for (i = 0; i < m->xxh->trk; i++)
-		free(m->xxt[i]);
-	for (i = 0; i < m->xxh->pat; i++)
-		free(m->xxp[i]);
-	for (i = 0; i < m->xxh->ins; i++) {
-		free(m->xxfe[i]);
-		free(m->xxpe[i]);
-		free(m->xxae[i]);
-		free(m->xxi[i]);
-	}
-	free(m->xxt);
-	free(m->xxp);
-	free(m->xxi);
-	if (m->xxh->smp > 0)
-		free(m->xxs);
-	free(m->xxim);
-	free(m->xxih);
-	free(m->xxfe);
-	free(m->xxpe);
-	free(m->xxae);
-	free(m->xxh);
-
-	if (m->dirname)
-		free(m->dirname);
-	if (m->basename)
-		free(m->basename);
+	return t;
 }
+#endif
+
 
 void xmp_get_driver_cfg(xmp_context ctx, int *srate, int *res, int *chn,
 			int *itpt)
@@ -285,7 +257,7 @@ int xmp_seek_time(xmp_context ctx, int time)
 			if (i > 0)
 				i--;
 			xmp_ord_set(ctx, i);
-			return XMP_OK;
+			return 0;
 		}
 	}
 	return -1;
