@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: load.c,v 1.69 2007/12/06 20:22:56 cmatsuoka Exp $
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -15,20 +13,32 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef __EMX__
+#include <ctype.h>
 #include <sys/types.h>
-#endif
 #include <sys/stat.h>
 #include <unistd.h>
+#include <limits.h>
+
+#if !defined(HAVE_POPEN) && defined(WIN32)
+#include "ptpopen.h"
+#endif
+
 #include "driver.h"
 #include "convert.h"
 #include "loader.h"
 
 #include "list.h"
-#include "../prowizard/prowiz.h"
+
+int pw_enable(char *, int);
 
 extern struct list_head loader_list;
-extern struct list_head *checked_format;
+
+LIST_HEAD(tmpfiles_list);
+
+struct tmpfilename {
+	char *name;
+	struct list_head list;
+};
 
 char *global_filename;
 
@@ -39,15 +49,12 @@ int decrunch_pp		(FILE *, FILE *);
 int decrunch_mmcmp	(FILE *, FILE *);
 int decrunch_oxm	(FILE *, FILE *);
 int decrunch_xfd	(FILE *, FILE *);
-int decrunch_pw		(FILE *, FILE *);
 int test_oxm		(FILE *);
 char *test_xfd		(unsigned char *, int);
-int pw_check		(unsigned char *, int);
 
 #define BUILTIN_PP	0x01
 #define BUILTIN_SQSH	0x02
 #define BUILTIN_MMCMP	0x03
-#define BUILTIN_PW	0x04
 #define BUILTIN_ARC	0x05
 #define BUILTIN_ARCFS	0x06
 #define BUILTIN_S404	0x07
@@ -55,8 +62,6 @@ int pw_check		(unsigned char *, int);
 #define BUILTIN_XFD	0x09
 
 
-#define TMP_SIZE 512
-
 #if defined __EMX__ || defined WIN32
 #define REDIR_STDERR "2>NUL"
 #elif defined unix
@@ -68,31 +73,36 @@ int pw_check		(unsigned char *, int);
 static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 {
     struct xmp_options *o = &ctx->o;
-    unsigned char *b;
+    unsigned char b[1024];
     char *cmd;
     FILE *t;
     int fd, builtin, res;
-    char *packer, *temp, *temp2, tmp[TMP_SIZE];
+    char *packer, *temp2, tmp[PATH_MAX];
+    struct tmpfilename *temp;
 
     packer = cmd = NULL;
     builtin = res = 0;
-    get_temp_dir(tmp, TMP_SIZE);
-    strncat(tmp, "xmp_XXXXXX", TMP_SIZE);
-    temp = strdup(tmp);
+    get_temp_dir(tmp, PATH_MAX);
+    strncat(tmp, "xmp_XXXXXX", PATH_MAX);
 
     fseek(*f, 0, SEEK_SET);
-    b = calloc(1, PW_TEST_CHUNK);
-    fread(b, 1, PW_TEST_CHUNK, *f);
+    fread(b, 1, 1024, *f);
 
 #if defined __AMIGA__ && !defined __AROS__
-    if (packer = test_xfd(b,PW_TEST_CHUNK)) {
+    if (packer = test_xfd(b, 1024)) {
 	builtin = BUILTIN_XFD;
     } else
 #endif
     if (b[0] == 'P' && b[1] == 'K') {
 	packer = "Zip";
-	cmd = "unzip -pqqC \"%s\" -x readme '*.diz' '*.nfo' '*.txt' "
-		"'*.exe' '*.com' " REDIR_STDERR;
+#if defined WIN32
+	cmd = "unzip -pqqC \"%s\" -x readme *.diz *.nfo *.txt *.exe *.com "
+		"README *.DIZ *.NFO *.TXT *.EXE *.COM " REDIR_STDERR;
+#else
+	cmd = "unzip -pqqC \"%s\" -x readme '*.diz' '*.nfo' '*.txt' '*.exe' "
+		"'*.com' README '*.DIZ' '*.NFO' '*.TXT' '*.EXE' '*.COM' "
+		REDIR_STDERR;
+#endif
     } else if (b[2] == '-' && b[3] == 'l' && b[4] == 'h') {
 	packer = "LHa";
 #if defined __EMX__
@@ -146,61 +156,63 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	packer = "oggmod";
 	builtin = BUILTIN_OXM;
 #endif
-    } else {
-	int extra;
-	int s = PW_TEST_CHUNK;
+    }
 
-	while ((extra = pw_check(b, s)) > 0) {
-	    b = realloc(b, s + extra);
-	    fread(b + s, extra, 1, *f);
-	    s += extra;
+    if (packer == NULL && b[0] == 0x1a) {
+	int x = b[1] & 0x7f;
+	int i, flag = 0;
+	long size;
+	
+	/* check file name */
+	for (i = 0; i < 13; i++) {
+	    if (b[2 + i] == 0) {
+		if (i == 0)		/* name can't be empty */
+		    flag = 1;
+		break;
+	    }
+	    if (!isprint(b[2 + i])) {	/* name must be printable */
+		flag = 1;
+		break;
+	    }
 	}
 
-	if (extra == 0) {
-	    struct pw_format *format;
+	size = readmem32l(b + 15);	/* max file size is 512KB */
+	if (size < 0 || size > 512 * 1024)
+		flag = 1;
 
-	    format = list_entry(checked_format, struct pw_format, list);
-
-	    if (format->enable) {
-	        packer = format->name;
-	        builtin = BUILTIN_PW;
-
-		xmp_enable_format("MOD", 1);
+        if (flag == 0) {
+	    if (x >= 1 && x <= 9 && x != 7) {
+		packer = "Arc";
+		builtin = BUILTIN_ARC;
+	    } else if (x == 0x7f) {
+		packer = "!Spark";
+		builtin = BUILTIN_ARC;
 	    }
 	}
     }
 
-    /* Test Arc after prowizard to prevent misidentification */
-    if (packer == NULL && b[0] == 0x1a) {
-	int x = b[1] & 0x7f;
-	if (x >= 1 && x <= 9 && x != 7) {
-	    packer = "Arc";
-	    builtin = BUILTIN_ARC;
-	} else if (x == 0x7f) {
-	    packer = "!Spark";
-	    builtin = BUILTIN_ARC;
-	}
-    }
-
-    free(b);
-
     fseek(*f, 0, SEEK_SET);
 
-    if (packer == NULL) {
-	free(temp);
+    if (packer == NULL)
 	return 0;
+
+    reportv(ctx, 0, "Depacking %s file... ", packer);
+
+    temp = calloc(sizeof (struct tmpfilename), 1);
+    if (!temp) {
+	report("calloc failed\n");
+	goto err;
     }
 
-    /* TODO: move pw to loaders -- add better Arc test above */
-    if (builtin != BUILTIN_PW)
-       reportv(ctx, 0, "Depacking %s file... ", packer);
-
-    if ((fd = mkstemp(temp)) < 0) {
+    temp->name = strdup(tmp);
+    if ((fd = mkstemp(temp->name)) < 0) {
 	if (o->verbosity > 0)
 	    report("failed\n");
 	goto err;
     }
 
+    list_add_tail(&temp->list, &tmpfiles_list);
+
     if ((t = fdopen(fd, "w+b")) == NULL) {
 	reportv(ctx, 0, "failed\n");
 	goto err;
@@ -216,14 +228,22 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	snprintf(line, lsize, cmd, *s);
 
 #ifdef WIN32
+	/* Note: The _popen function returns an invalid file handle, if
+	 * used in a Windows program, that will cause the program to hang
+	 * indefinitely. _popen works properly in a Console application.
+	 * To create a Windows application that redirects input and output,
+	 * read the section "Creating a Child Process with Redirected Input
+	 * and Output" in the Win32 SDK. -- Mirko 
+	 */
 	if ((p = popen(line, "rb")) == NULL) {
 #else
+	/* Linux popen fails with "rb" */
 	if ((p = popen(line, "r")) == NULL) {
 #endif
 	    reportv(ctx, 0, "failed\n");
 	    fclose(t);
 	    free(line);
-	    goto err2;
+	    goto err;
 	}
 	free (line);
 #define BSIZE 0x4000
@@ -231,7 +251,7 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	    reportv(ctx, 0, "failed\n");
 	    pclose (p);
 	    fclose (t);
-	    goto err2;
+	    goto err;
 	}
 	while ((n = fread(buf, 1, BSIZE, p)) > 0) {
 	    fwrite(buf, 1, n, t);
@@ -255,7 +275,7 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	case BUILTIN_MMCMP:    
 	    res = decrunch_mmcmp(*f, t);
 	    break;
-#if !defined __MINGW32__ && !defined __AMIGA__
+#if !defined WIN32 && !defined __MINGW32__ && !defined __AMIGA__
 	case BUILTIN_OXM:
 	    res = decrunch_oxm(*f, t);
 	    break;
@@ -265,15 +285,12 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
 	    res = decrunch_xfd(*f, t);
 	    break;
 #endif
-	case BUILTIN_PW:
-	    res = decrunch_pw(*f, t);
-	    break;
 	}
     }
 
     if (res < 0) {
 	reportv(ctx, 0, "failed\n");
-	goto err2;
+	goto err;
     }
 
     reportv(ctx, 0, "done\n");
@@ -281,22 +298,46 @@ static int decrunch(struct xmp_context *ctx, FILE **f, char **s)
     fclose(*f);
     *f = t;
  
-    temp2 = strdup(temp);
-    decrunch(ctx, f, &temp);
+    temp2 = strdup(temp->name);
+    decrunch(ctx, f, &temp->name);
     unlink(temp2);
     free(temp2);
-    free(temp);
+    /* Mirko: temp is now deallocated in xmp_unlink_tempfiles
+     * not a problem, since xmp_unlink_tempfiles is called after decrunch
+     * in loader routines
+     *
+     * free(temp);
+     */
 
     return 1;
 
-err2:
-    unlink(temp);
 err:
-    free(temp);
     return -1;
 }
 
 
+/*
+ * Windows doesn't allow you to unlink an open file, so we changed the
+ * temp file cleanup system to remove temporary files after we close it
+ */
+void xmp_unlink_tempfiles(void)
+{
+	struct tmpfilename *li;
+	struct list_head *tmp;
+
+	/* can't use list_for_each when freeing the node! */
+	for (tmp = (&tmpfiles_list)->next; tmp != (&tmpfiles_list); ) {
+		li = list_entry(tmp, struct tmpfilename, list);
+		_D(_D_INFO "unlink tmpfile %s", li->name);
+		unlink(li->name);
+		free(li->name);
+		list_del(&li->list);
+		tmp = tmp->next;
+		free(li);
+	}
+}
+
+
 static void get_smp_size(struct xmp_player_context *p, int awe, int *a, int *b)
 {
     int i, len, smp_size, smp_4kb;
@@ -332,7 +373,9 @@ static int crunch_ratio(struct xmp_context *ctx, int awe)
     int memavl, smp_size, ratio, smp_4kb;
 
     ratio = 0x10000;
-    if (!(memavl = d->memavl))
+    memavl = d->memavl;
+
+    if (memavl == 0)
 	return ratio;
 
     memavl = memavl * 100 / (100 + o->crunch);
@@ -347,7 +390,7 @@ static int crunch_ratio(struct xmp_context *ctx, int awe)
 
     if (smp_size > memavl) {
 	ratio = (int)
-	    (((long long)(memavl - smp_4kb) << 16) / (smp_size - smp_4kb));
+	    (((int64)(memavl - smp_4kb) << 16) / (smp_size - smp_4kb));
 	if (o->verbosity)
 	    report ("Crunch ratio   : %d%% [Mem:%.3fMb Smp:%.3fMb]\n",
 		100 - 100 * ratio / 0x10000, .000001 * d->memavl,
@@ -386,6 +429,7 @@ int xmp_test_module(xmp_context ctx, char *s, char *n)
 	    fseek(f, 0, SEEK_SET);
 	    if (li->test(f, n, 0) == 0) {
 	        fclose(f);
+		xmp_unlink_tempfiles();
 	        return 0;
 	    }
 	}
@@ -401,6 +445,7 @@ static void split_name(char *s, char **d, char **b)
 {
 	char tmp, *div;
 
+	_D("alloc dirname/basename");
 	if ((div = strrchr(s, '/'))) {
 		tmp = *(div + 1);
 		*(div + 1) = 0;
@@ -421,11 +466,11 @@ int xmp_load_module(xmp_context ctx, char *s)
     struct xmp_loader_info *li;
     struct list_head *head;
     struct stat st;
-    unsigned int crc;
     struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
     struct xmp_driver_context *d = &((struct xmp_context *)ctx)->d;
     struct xmp_mod_context *m = &p->m;
     struct xmp_options *o = &((struct xmp_context *)ctx)->o;
+    uint32 crc = 0;
 
     _D(_D_WARN "s = %s", s);
 
@@ -447,8 +492,6 @@ int xmp_load_module(xmp_context ctx, char *s)
 
     split_name(s, &m->dirname, &m->basename);
 
-    crc = cksum(f);
-
     _D(_D_INFO "clear mem");
     xmp_drv_clearmem((struct xmp_context *)ctx);
 
@@ -465,9 +508,7 @@ int xmp_load_module(xmp_context ctx, char *s)
     m->vol_xlat = NULL;
     /* Reset control for next module */
     m->fetch = o->flags & ~XMP_CTL_FILTER;
-
-    _D(_D_INFO "read modconf");
-    xmpi_read_modconf((struct xmp_context *)ctx, crc, st.st_size);
+    m->comment = NULL;
 
     m->xxh = calloc(sizeof (struct xxm_header), 1);
     /* Set defaults */
@@ -477,8 +518,6 @@ int xmp_load_module(xmp_context ctx, char *s)
 
     for (i = 0; i < 64; i++) {
 	m->xxc[i].pan = (((i + 1) / 2) % 2) * 0xff;
-	m->xxc[i].cho = o->chorus;
-	m->xxc[i].rvb = o->reverb;
 	m->xxc[i].vol = 0x40;
 	m->xxc[i].flg = 0;
     }
@@ -502,15 +541,29 @@ int xmp_load_module(xmp_context ctx, char *s)
 		report("Identified as %s\n", li->id);
 	    fseek(f, 0, SEEK_SET);
 	    _D(_D_WARN "load format: %s (%s)", li->id, li->name);
-	    if ((i = li->loader((struct xmp_context *)ctx, f, 0) == 0))
+	    if ((i = li->loader((struct xmp_context *)ctx, f, 0) == 0)) {
+		crc = cksum(f);
 		break;
+	    }
 	}
     }
 
     fclose(f);
+    xmp_unlink_tempfiles();
 
-    if (i < 0)
+    if (i < 0) {
+	free(m->basename);
+	free(m->dirname);
+	free(m->xxh);
 	return i;
+    }
+
+    _xmp_read_modconf((struct xmp_context *)ctx, crc, st.st_size);
+
+    for (i = 0; i < 64; i++) {
+	m->xxc[i].cho = o->chorus;	/* set after reading modconf */
+	m->xxc[i].rvb = o->reverb;
+    }
 
     if (d->description && (i = (strstr(d->description, " [AWE") != NULL))) {
 	xmp_cvt_to16bit((struct xmp_context *)ctx);
@@ -541,6 +594,7 @@ int xmp_load_module(xmp_context ctx, char *s)
     }
 
     if (o->verbosity > 2) {
+	char * amp_factor[] = { "Normal", "x2", "x4", "x8" };
 	report("Amiga range    : %s\n", m->xxh->flg & XXM_FLG_MODRNG ?
 		"yes" : "no");
 	report("Restart pos    : %d\n", m->xxh->rst);
@@ -549,12 +603,16 @@ int xmp_load_module(xmp_context ctx, char *s)
 	report("Channel mixing : %d%% (dynamic pan %s)\n",
 		m->fetch & XMP_CTL_REVERSE ? -o->mix : o->mix,
 		m->fetch & XMP_CTL_DYNPAN ? "enabled" : "disabled");
+	report("Checksum       : %u %ld\n", crc, st.st_size);
+	report("Volume amplify : %s\n", amp_factor[o->amplify]);
     }
 
     if (o->verbosity) {
 	report("Channels       : %d [ ", m->xxh->chn);
 	for (i = 0; i < m->xxh->chn; i++) {
-	    if (m->xxc[i].flg & XXM_CHANNEL_FM)
+	    if (m->xxc[i].flg & XXM_CHANNEL_MUTE)
+		report("- ");
+	    else if (m->xxc[i].flg & XXM_CHANNEL_FM)
 		report("F ");
 	    else
 	        report("%x ", m->xxc[i].pan >> 4);
@@ -562,7 +620,7 @@ int xmp_load_module(xmp_context ctx, char *s)
 	report("]\n");
     }
 
-    t = xmpi_scan_module((struct xmp_context *)ctx);
+    t = _xmp_scan_module((struct xmp_context *)ctx);
 
     if (o->verbosity) {
 	if (m->fetch & XMP_CTL_LOOP)
@@ -579,6 +637,7 @@ int xmp_load_module(xmp_context ctx, char *s)
 
 err:
     fclose(f);
+    xmp_unlink_tempfiles();
     return -1;
 }
 
@@ -599,3 +658,57 @@ int xmp_enable_format(char *id, int enable)
     return pw_enable(id, enable);
 }
 
+
+void xmp_release_module(xmp_context ctx)
+{
+	struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
+	struct xmp_mod_context *m = &p->m;
+	int i;
+
+	_D(_D_INFO "Freeing memory");
+
+	if (m->med_vol_table) {
+		for (i = 0; i < m->xxh->ins; i++)
+			if (m->med_vol_table[i])
+				free(m->med_vol_table[i]);
+		free(m->med_vol_table);
+	}
+
+	if (m->med_wav_table) {
+		for (i = 0; i < m->xxh->ins; i++)
+			if (m->med_wav_table[i])
+				free(m->med_wav_table[i]);
+		free(m->med_wav_table);
+	}
+
+	for (i = 0; i < m->xxh->trk; i++)
+		free(m->xxt[i]);
+
+	for (i = 0; i < m->xxh->pat; i++)
+		free(m->xxp[i]);
+
+	for (i = 0; i < m->xxh->ins; i++) {
+		free(m->xxfe[i]);
+		free(m->xxpe[i]);
+		free(m->xxae[i]);
+		free(m->xxi[i]);
+	}
+
+	free(m->xxt);
+	free(m->xxp);
+	free(m->xxi);
+	if (m->xxh->smp > 0)
+		free(m->xxs);
+	free(m->xxim);
+	free(m->xxih);
+	free(m->xxfe);
+	free(m->xxpe);
+	free(m->xxae);
+	free(m->xxh);
+	if (m->comment)
+		free(m->comment);
+
+	_D("free dirname/basename");
+	free(m->dirname);
+	free(m->basename);
+}
