@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr.
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr.
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See docs/COPYING
  * for more information.
- *
- * $Id: driver.c,v 1.74 2007/11/25 13:58:10 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,8 +44,8 @@ extern struct xmp_drv_info drv_openbsd;
 extern struct xmp_drv_info drv_sgi;
 extern struct xmp_drv_info drv_aix;
 extern struct xmp_drv_info drv_oss_seq;
-extern struct xmp_drv_info drv_oss_mix;
-extern struct xmp_drv_info drv_alsa_mix;
+extern struct xmp_drv_info drv_oss;
+extern struct xmp_drv_info drv_alsa;
 extern struct xmp_drv_info drv_alsa05;
 extern struct xmp_drv_info drv_net;
 extern struct xmp_drv_info drv_esd;
@@ -101,11 +99,11 @@ void xmp_init_drivers()
 #ifdef DRIVER_OSS_SEQ
     xmp_drv_register(&drv_oss_seq);
 #endif
-#ifdef DRIVER_OSS_MIX
-    xmp_drv_register(&drv_oss_mix);
+#ifdef DRIVER_OSS
+    xmp_drv_register(&drv_oss);
 #endif
-#ifdef DRIVER_ALSA_MIX
-    xmp_drv_register(&drv_alsa_mix);
+#ifdef DRIVER_ALSA
+    xmp_drv_register(&drv_alsa);
 #endif
 #ifdef DRIVER_ALSA05
     xmp_drv_register(&drv_alsa05);
@@ -156,7 +154,7 @@ static int drv_select(struct xmp_context *ctx)
  	ret = XMP_ERR_NODRV;
 	for (drv = drv_array; drv; drv = drv->next) {
 	    if (!strcmp (drv->id, o->drv_id)) {
-		if ((ret = drv->init(ctx)) == XMP_OK)
+		if ((ret = drv->init(ctx)) == 0)
 		    break;
 	    }
 	}
@@ -167,17 +165,17 @@ static int drv_select(struct xmp_context *ctx)
 	for (; drv; drv = drv->next) {
 	    if (o->verbosity > 2)
 		report ("Probing %s... ", drv->description);
-	    if (drv->init(ctx) == XMP_OK) {
+	    if (drv->init(ctx) == 0) {
 		if (o->verbosity > 2)
 		    report ("found\n");
-		ret = XMP_OK;
+		ret = 0;
 		break;
 	    }
 	    if (o->verbosity > 2)
 		report ("not found\n");
 	}
     }
-    if (ret != XMP_OK)
+    if (ret)
 	return ret;
 
     o->drv_id = drv->id;
@@ -185,7 +183,7 @@ static int drv_select(struct xmp_context *ctx)
     d->help = drv->help;
     d->driver = drv;
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -231,7 +229,7 @@ int xmp_drv_open(struct xmp_context *ctx)
     d->memavl = 0;
     smix_buf32b = NULL;
     extern_drv = TURN_ON;
-    if ((status = drv_select(ctx)) != XMP_OK)
+    if ((status = drv_select(ctx)) != 0)
 	return status;
 
     d->patch_array = calloc(XMP_MAXPAT, sizeof(struct patch_info *));
@@ -244,7 +242,7 @@ int xmp_drv_open(struct xmp_context *ctx)
     synth_init(o->freq);
     synth_reset();
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -261,7 +259,7 @@ int xmp_drv_set(struct xmp_context *ctx)
     for (drv = drv_array; drv; drv = drv->next) {
 	if (!strcmp (drv->id, o->drv_id)) {
 	    d->driver = drv;
-	    return XMP_OK;
+	    return 0;
 	}
     }
 
@@ -320,7 +318,7 @@ int xmp_drv_on(struct xmp_context *ctx, int num)
     smix_resol = o->resol > 8 ? 2 : 1;
     smix_resetvar(ctx);
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -663,8 +661,8 @@ void xmp_drv_voicepos(struct xmp_context *ctx, int chn, int pos)
     pi = d->patch_array[d->voice_array[voc].smp];
 
     if (pi->base_note != C4_FREQ)	/* process crunching samples */
-	pos = ((long long)pos << SMIX_SHIFT) / (int)
-	    (((long long)pi->base_note << SMIX_SHIFT) / C4_FREQ);
+	pos = ((int64)pos << SMIX_SHIFT) / (int)
+	    (((int64)pi->base_note << SMIX_SHIFT) / C4_FREQ);
 
     if (pos > pi->len)	/* Attempt to set offset beyond the end of sample */
 	return;
@@ -767,8 +765,8 @@ int xmp_drv_writepatch(struct xmp_context *ctx, struct patch_info *patch)
     struct xmp_driver_context *d = &ctx->d;
     int num;
 
-    if (!d->patch_array)	/* FIXME -- this makes xmms happy */
-    	return XMP_OK;
+    if (!d->patch_array)
+    	return 0;
 
     if (!patch) {
 	d->driver->writepatch(ctx, patch);
@@ -777,13 +775,13 @@ int xmp_drv_writepatch(struct xmp_context *ctx, struct patch_info *patch)
 	    free(d->patch_array[num]);
 	    d->patch_array[num] = NULL;
 	}
-	return XMP_OK;
+	return 0;
     }
     if (patch->instr_no >= XMP_MAXPAT)
 	return XMP_ERR_PATCH;
     d->patch_array[patch->instr_no] = patch;
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -795,7 +793,7 @@ int xmp_drv_flushpatch(struct xmp_context *ctx, int ratio)
     int smp, num, crunch;
 
     if (!d->patch_array)		/* FIXME -- this makes xmms happy */
-	return XMP_OK;
+	return 0;
 
     if (!ratio)
 	ratio = 0x10000;
@@ -819,14 +817,14 @@ int xmp_drv_flushpatch(struct xmp_context *ctx, int ratio)
 
 	    crunch = xmp_cvt_crunch(&patch, ratio);
 	    xmp_cvt_anticlick (patch);
-	    if ((num = d->driver->writepatch(ctx, patch)) != XMP_OK) {
+	    if ((num = d->driver->writepatch(ctx, patch)) != 0) {
 		d->patch_array[smp] = NULL;	/* Bad type, reset array */
 		free (patch);
 	    } else 
 		d->patch_array[smp] = realloc(patch, sizeof (struct patch_info));
 
 	    if (o->verbosity) {
-		if (num != XMP_OK)
+		if (num)
 		    report ("E");		/* Show type error */
 		else if (!crunch)
 		    report ("i");		/* Show sbi patch type */
@@ -841,14 +839,14 @@ int xmp_drv_flushpatch(struct xmp_context *ctx, int ratio)
 		continue;
 	    patch = d->patch_array[smp];
 	    xmp_cvt_anticlick (patch);
-	    if (d->driver->writepatch(ctx, patch) != XMP_OK) {
+	    if (d->driver->writepatch(ctx, patch) != 0) {
 		d->patch_array[smp] = NULL;	/* Bad type, reset array */
 		free (patch);
 	    }
 	}
     }
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -894,7 +892,7 @@ int xmp_drv_loadpatch(struct xmp_context *ctx, FILE *f, int id, int basefreq, in
     if (o->skipsmp) {		/* Will fail for ADPCM samples */
         if (~flags & XMP_SMP_NOLOAD)
 	    fseek(f, xxs->len, SEEK_CUR);
-	return XMP_OK;
+	return 0;
     }
 
     /* Empty samples
@@ -902,7 +900,7 @@ int xmp_drv_loadpatch(struct xmp_context *ctx, FILE *f, int id, int basefreq, in
     if (xxs->len < 4) {
 	if (~flags & XMP_SMP_NOLOAD)
 	    fread(s, 1, xxs->len, f);
-	return XMP_OK;
+	return 0;
     }
     /* Patches with samples
      */
@@ -937,7 +935,7 @@ int xmp_drv_loadpatch(struct xmp_context *ctx, FILE *f, int id, int basefreq, in
 	    xmp_cvt_sex(xxs->len, patch->data);
     }
 
-    if (xxs->flg & WAVE_STEREO) {	/* Downmix stereo samples */
+    if (flags & XMP_SMP_STEREO) {	/* Downmix stereo samples */
 	xmp_cvt_stdownmix(xxs->len, xxs->flg & WAVE_16_BITS, patch->data);
 	xxs->len /= 2;
     }
