@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -11,7 +11,7 @@
 #define DO_TONEPORTA() { \
 	if (note-- && note < 0x60 && (uint32)xc->ins < p->m.xxh->ins) \
 	    xc->s_end = note_to_period (note + XXI[XXIM.ins[note]].xpo + \
-		XXIM.xpo[note] /*, XXI[XXIM.ins[note]].fin*/, \
+		XXIM.xpo[note], XXI[XXIM.ins[note]].fin, \
 	 	p->m.xxh->flg & XXM_FLG_LINEAR); \
 	xc->s_sgn = xc->period < xc->s_end ? 1 : -1; \
 }
@@ -31,6 +31,7 @@ void process_fx(struct xmp_context *ctx, int chn, uint8 note, uint8 fxt, uint8 f
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_mod_context *m = &p->m;
+    struct flow_control *f = &p->flow;
     int h, l;
 
     switch (fxt) {
@@ -120,8 +121,11 @@ fx_porta_dn:
 	if (!TEST (IS_VALID))
 	    break;
 	DO_TONEPORTA();
-	if (fxp)
+	if (fxp) {
 	    xc->s_val = fxp;
+	    if (m->fetch & XMP_CTL_UNISLD)	/* IT compatible Gxx off */
+		xc->porta = fxp;
+	}
 	SET(TONEPORTA);
 	break;
     case FX_PER_PORTA_UP:			/* Persistent portamento up */
@@ -222,40 +226,52 @@ fx_porta_dn:
 	    xc->offset_val = xc->offset;
 	break;
     case FX_VOLSLIDE:				/* Volume slide */
+	/* S3M file volume slide note:
+	 * DFy	Fine volume down by y (...) If y is 0, the command will be
+	 *	treated as a volume slide up with a value of f (15). If a
+	 *	DFF command is specified, the volume will be slid up.
+	 */
 fx_volslide:
 	if (m->fetch & XMP_CTL_FINEFX) {
 	    h = MSN(fxp);
 	    l = LSN(fxp);
-	    if (h == 0xf && l != 0) {
-		xc->volslide = fxp;
-		fxp &= 0x0f;
-		goto ex_f_vslide_dn;
-	    } else if (h == 0xe && l != 0) {
-		xc->volslide = fxp;
-		fxp &= 0x0f;
-		goto ex_f_vslide_dn;		/* FIXME */
-	    } else if (l == 0xf && h != 0) {
+	    if (l == 0xf && h != 0) {
 		xc->volslide = fxp;
 		fxp >>= 4;
 		goto ex_f_vslide_up;
-	    } else if (l == 0xe && h != 0) {
+	    } else if (h == 0xf && l != 0) {
 		xc->volslide = fxp;
-		fxp >>= 4;
-		goto ex_f_vslide_up;		/* FIXME */
+		fxp &= 0x0f;
+		goto ex_f_vslide_dn;
 	    } else if (!fxp) {
 		if ((fxp = xc->volslide) != 0)
 		    goto fx_volslide;
 	    }
 	}
 	SET(VOL_SLIDE);
+
 	/* Skaven's 2nd reality (S3M) has volslide parameter D7 => pri down
 	 * Stargazer's Red Dream (MOD) uses volslide DF => compute both
+	 * Also don't assign xc->volslide if fxp is 0, see Guild of Sounds.xm
 	 */
-	if (m->fetch & XMP_CTL_VOLPDN) {
-	    if ((xc->volslide = fxp))
-	        xc->v_val = LSN(fxp) ? -LSN(fxp) : MSN(fxp);
-	} else {
-	    xc->v_val = -LSN(fxp) + MSN(fxp);
+	if (fxp) {
+	    if (m->fetch & XMP_CTL_VOLPDN) {
+		if ((xc->volslide = fxp))
+	            xc->v_val = LSN(fxp) ? -LSN(fxp) : MSN(fxp);
+	    } else {
+		xc->v_val = -LSN(fxp) + MSN(fxp);
+	    }
+	}
+
+	if (m->fetch & XMP_CTL_FINEFX) {
+	    /* Mirko reports that a S3M with D0F effects created with ST321
+	     * should process volume slides in all frames like ST300. I
+	     * suspect ST3/IT could be handling D0F effects like this.
+	     */
+	    if (MSN(xc->volslide) == 0xf || LSN(xc->volslide) == 0xf) {
+        	SET(FINE_VOLS);
+        	xc->v_fval = xc->v_val;
+	    }
 	}
 	break;
     case FX_VOLSLIDE_2:				/* Secondary volume slide */
@@ -264,11 +280,13 @@ fx_volslide:
 	    xc->v_val2 = -LSN(fxp) + MSN(fxp);
 	break;
     case FX_VOLSLIDE_UP:			/* Vol slide with uint8 arg */
-	xc->v_val = fxp;
+	if (fxp)
+	    xc->v_val = fxp;
 	SET(VOL_SLIDE);
 	break;
     case FX_VOLSLIDE_DN:			/* Vol slide with uint8 arg */
-	xc->v_val = -fxp;
+	if (fxp)
+	    xc->v_val = -fxp;
 	SET(VOL_SLIDE);
 	break;
     case FX_F_VSLIDE:				/* Fine volume slide */
@@ -319,21 +337,24 @@ ex_f_porta_dn:
 	    fxp <<= 4;
 	    goto fx_finetune;
 	case EX_PATTERN_LOOP:			/* Loop pattern */
-	    if (fxp) {
-		if (p->flow.loop_stack[chn]) {
-		    if (--p->flow.loop_stack[chn])
-			p->flow.loop_chn = ++chn;	/* **** H:FIXME **** */
-		    else
+	    if (fxp == 0) {
+		/* mark start of loop */
+		f->loop_start[chn] = p->flow.row;
+	    } else {
+		/* end of loop */
+		if (f->loop_stack[chn]) {
+		    if (--f->loop_stack[chn]) {
+			f->loop_chn = ++chn;	/* **** H:FIXME **** */
+		    } else {
 			if (m->fetch & XMP_CTL_S3MLOOP)
-			    p->flow.loop_row[chn] = p->flow.row_cnt + 1;
+			    f->loop_start[chn] = f->row + 1;
+		    }
 		} else {
-		    if (p->flow.loop_row[chn] <= p->flow.row_cnt) {
-			p->flow.loop_stack[chn] = fxp;
-			p->flow.loop_chn = ++chn;
+		    if (f->loop_start[chn] <= f->row) {
+			f->loop_stack[chn] = fxp;
+			f->loop_chn = ++chn;
 		    }
 		}
-	    } else {
-		p->flow.loop_row[chn] = p->flow.row_cnt;
 	    }
 	    break;
 	case EX_TREMOLO_WF:			/* Set tremolo waveform */
@@ -372,7 +393,7 @@ ex_f_vslide_dn:
 	if (fxp) {
 	    if (fxp < 0x20)	/* speedup.xm needs BPM = 20 */
 		p->tempo = fxp;
-	    else
+	    else if (~m->fetch & XMP_CTL_VBLANK)
 		p->tick_time = m->rrate / (p->xmp_bpm = fxp);
 	}
 	break;
@@ -384,6 +405,20 @@ ex_f_vslide_dn:
 	if (fxp >= 0x20)	/* Panic uses 0x14 */
 	    p->tick_time = m->rrate / (p->xmp_bpm = fxp);
 	break;
+    case FX_IT_BPM:				/* Set IT BPM */
+	if (MSN(fxp) == 0) {		/* T0x - Tempo slide down by x */
+	    p->xmp_bpm -= LSN(fxp);
+	    if (p->xmp_bpm < 0x20)
+		p->xmp_bpm = 0x20;
+	} else if (MSN(fxp) == 1) {	/* T1x - Tempo slide up by x */
+	    p->xmp_bpm += LSN(fxp);
+	    if ((int32)p->xmp_bpm > 0xff)
+		p->xmp_bpm = 0xff;
+	} else {
+	    p->xmp_bpm = fxp;
+	}
+	p->tick_time = m->rrate / p->xmp_bpm;
+	break;
     case FX_GLOBALVOL:				/* Set global volume */
 	m->volume = fxp > m->volbase ? m->volbase : fxp;
 	break;
