@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: formats.c,v 1.68 2007/11/20 23:38:19 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -13,7 +11,7 @@
 #endif
 
 #include <stdlib.h>
-#include "xmpi.h"
+#include "common.h"
 #include "list.h"
 #include "loader.h"
 
@@ -21,6 +19,7 @@ struct xmp_fmt_info *__fmt_head;
 
 LIST_HEAD(loader_list);
 
+extern struct xmp_loader_info pw_loader;
 
 void register_format(char *id, char *tracker)
 {
@@ -71,8 +70,8 @@ void xmp_init_formats(xmp_context ctx)
 	REG_LOADER(ult);
 	REG_LOADER(liq);
 	REG_LOADER(no);
+	REG_LOADER(masi);
 	REG_LOADER(psm);
-	REG_LOADER(svb);
 	REG_LOADER(amf);
 	REG_LOADER(gdm);
 	REG_LOADER(mmd1);
@@ -97,7 +96,7 @@ void xmp_init_formats(xmp_context ctx)
 	REG_LOADER(far);
 	REG_LOADER(umx);
 	REG_LOADER(stim);
-	//REG_LOADER(coco);
+	/* REG_LOADER(coco); */
 	REG_LOADER(mtp);
 	REG_LOADER(ims);
 	REG_LOADER(ssn);
@@ -106,4 +105,7 @@ void xmp_init_formats(xmp_context ctx)
 	REG_LOADER(rad);
 	REG_LOADER(hsc);
 	REG_LOADER(alm);
+
+	pw_loader.enable = 1;
+	list_add_tail(&pw_loader.list, &loader_list);
 }
