@@ -1,7 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1997-2007 Claudio Matsuoka and Hipolito Carraro Jr
- *
- * $Id: mixer.c,v 1.35 2007/11/30 02:11:27 cmatsuoka Exp $
+ * Copyright (C) 1997-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -26,7 +24,7 @@
 #define FLAG_SYNTH	0x40
 #define FIDX_FLAGMASK	(FLAG_ITPT | FLAG_16_BITS | FLAG_STEREO | FLAG_FILTER)
 
-#define LIM_FT		 12
+#define DOWNMIX_SHIFT	 12
 #define LIM8_HI		 127
 #define LIM8_LO		-127
 #define LIM12_HI	 4095
@@ -42,7 +40,7 @@ static int* smix_buf32b = NULL;		/* temp buffer for 32 bit samples */
 static int smix_numvoc;			/* default softmixer voices number */
 static int smix_numbuf;			/* number of buffer to output */
 static int smix_mode;			/* mode = 0:OFF, 1:MONO, 2:STEREO */
-static int smix_resol;			/* resolution output 0:8bit, 1:16bit */
+static int smix_resol;			/* resolution output 1:8bit, 2:16bit */
 static int smix_ticksize;
 int **xmp_mix_buffer = &smix_buf32b;
 
@@ -67,9 +65,9 @@ void    smix_st16itpt_flt (struct voice_info *, int *, int, int, int, int);
 
 void    smix_synth	  (struct voice_info *, int *, int, int, int, int);
 
-static void     out_su8norm     (char*, int*, int, int);
-static void     out_su16norm    (int16*, int*, int, int);
-static void     out_u8ulaw      (char*, int*, int, int);
+static void     out_su8norm     (char*, int*, int, int, int);
+static void     out_su16norm    (int16*, int*, int, int, int);
+static void     out_u8ulaw      (char*, int*, int, int, int);
 
 /* Array index:
  *
@@ -111,46 +109,49 @@ static void (*out_fn[])() = {
 };
 
 /* Downmix 32bit samples to 8bit, signed or unsigned, mono or stereo output */
-static void out_su8norm(char *dest, int *src, int num, int cod)
+static void out_su8norm(char *dest, int *src, int num, int amp, int flags)
 {
     int smp, lhi, llo, offs;
+    int shift = DOWNMIX_SHIFT + 8 - amp;
 
-    offs = (cod & XMP_FMT_UNS) ? 0x80 : 0;
+    offs = (flags & XMP_FMT_UNS) ? 0x80 : 0;
 
     lhi = LIM8_HI + offs;
     llo = LIM8_LO + offs;
 
     for (; num--; ++src, ++dest) {
-	smp = *src >> (LIM_FT + 8);
+	smp = *src >> shift;
 	*dest = smp > LIM8_HI ? lhi : smp < LIM8_LO ? llo : smp + offs;
     }
 }
 
 
 /* Downmix 32bit samples to 16bit, signed or unsigned, mono or stereo output */
-static void out_su16norm(int16 *dest, int *src, int num, int cod)
+static void out_su16norm(int16 *dest, int *src, int num, int amp, int flags)
 {
     int smp, lhi, llo, offs;
+    int shift = DOWNMIX_SHIFT - amp;
 
-    offs = (cod & XMP_FMT_UNS) ? 0x8000 : 0;
+    offs = (flags & XMP_FMT_UNS) ? 0x8000 : 0;
 
     lhi = LIM16_HI + offs;
     llo = LIM16_LO + offs;
 
     for (; num--; ++src, ++dest) {
-	smp = *src >> LIM_FT;
+	smp = *src >> shift;
 	*dest = smp > LIM16_HI ? lhi : smp < LIM16_LO ? llo : smp + offs;
     }
 }
 
 
 /* Downmix 32bit samples to 8bit, unsigned ulaw, mono or stereo output */
-static void out_u8ulaw(char *dest, int *src, int num, int cod)
+static void out_u8ulaw(char *dest, int *src, int num, int amp, int flags)
 {
     int smp;
+    int shift = DOWNMIX_SHIFT + 4 - amp;
 
     for (; num--; ++src, ++dest) {
-	smp = *src >> (LIM_FT + 4);
+	smp = *src >> shift;
 	*dest = smp > LIM12_HI ? ulaw_encode(LIM12_HI) :
 		smp < LIM12_LO ? ulaw_encode(LIM12_LO) : ulaw_encode (smp);
     }
@@ -183,6 +184,7 @@ static void smix_rampdown(struct xmp_context *ctx, int voc, int32 *buf, int cnt)
     int dec_l, dec_r;
 
     if (voc < 0) {
+	/* initialize */
 	smp_r = smix_dtright;
 	smp_l = smix_dtleft;
     } else {
@@ -196,7 +198,7 @@ static void smix_rampdown(struct xmp_context *ctx, int voc, int32 *buf, int cnt)
 
     if (!buf) {
 	buf = smix_buf32b;
-	cnt = 16; //smix_ticksize;
+	cnt = SLOW_RELEASE; //smix_ticksize;
     }
     if (!cnt)
 	return;
@@ -289,8 +291,8 @@ static int softmixer(struct xmp_context *ctx)
 	}
 
 	buf_pos = smix_buf32b;
-	vol_r = (vi->vol * (0x80 - vi->pan));
-	vol_l = (vi->vol * (0x80 + vi->pan));
+	vol_r = vi->vol * (0x80 - vi->pan);
+	vol_l = vi->vol * (0x80 + vi->pan);
 
 	if (vi->fidx & FLAG_SYNTH) {
 	    if (synth) {
@@ -301,7 +303,10 @@ static int softmixer(struct xmp_context *ctx)
 	    continue;
 	}
 
-	itp_inc = ((long long)vi->pbase << SMIX_SHIFT) / vi->period;
+	itp_inc = ((int64)vi->pbase << SMIX_SHIFT) / vi->period;
+
+	if (itp_inc == 0)	/* otherwise m5v-nwlf.t crashes */
+	    continue;
 
 	pi = d->patch_array[vi->smp];
 
@@ -328,7 +333,7 @@ static int softmixer(struct xmp_context *ctx)
 	for (tic_cnt = smix_ticksize; tic_cnt; ) {
 	    /* How many samples we can write before the loop break or
 	     * sample end... */
-	    smp_cnt = 1 + (((long long)(vi->end - vi->pos) << SMIX_SHIFT)
+	    smp_cnt = 1 + (((int64)(vi->end - vi->pos) << SMIX_SHIFT)
 		- vi->itpt) / itp_inc;
 
 	    if (itp_inc > 0) {
@@ -396,9 +401,6 @@ static int softmixer(struct xmp_context *ctx)
 		continue;
 	    }
 
-	    /* FIXME: short samples with bidirectional loops still not ok
-	     *        test with jt_xmas.xm
-	     */
 	    if ((~vi->fidx & FLAG_REVLOOP) && vi->fxor == 0) {
 		vi->pos -= lpe - lps;			/* forward loop */
 		if (pi->mode & WAVE_PTKLOOP) {
@@ -463,7 +465,7 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
 
     vi->smp = smp;
     vi->vol = 0;
-    vi->freq = (long long) C4_FREQ * pi->base_freq / o->freq;
+    vi->freq = (int64)C4_FREQ * pi->base_freq / o->freq;
     
     if (pi->len == XMP_PATCH_FM) {
 	vi->fidx = FLAG_SYNTH;
@@ -502,6 +504,9 @@ static void smix_setpatch(struct xmp_context *ctx, int voc, int smp)
     else
 	vi->fxor = vi->fidx;
 
+    if (o->cf_cutoff)			/* Filter-based anticlick */
+	vi->fidx |= FLAG_FILTER;
+
     if (pi->mode & WAVE_PTKLOOP)
 	pi->mode |= WAVE_FIRSTRUN;
 
@@ -516,6 +521,7 @@ static void smix_setnote(struct xmp_context *ctx, int voc, int note)
 
     vi->period = note_to_period_mix(vi->note = note, 0);
     vi->pbase = SMIX_C4NOTE * vi->freq / d->patch_array[vi->smp]->base_note;
+    vi->attack = SLOW_ATTACK;
 }
 
 
@@ -538,6 +544,9 @@ void xmp_smix_setvol(struct xmp_context *ctx, int voc, int vol)
  
     smix_anticlick(ctx, voc, vol, vi->pan, NULL, 0);
     vi->vol = vol;
+
+    if (vi->fidx & FLAG_SYNTH)
+	synth_setvol(voc, vol >> 4);
 }
 
 
@@ -599,7 +608,7 @@ int xmp_smix_writepatch(struct xmp_context *ctx, struct patch_info *patch)
 {
     if (patch) {
 	if (patch->len == XMP_PATCH_FM)
-	    return XMP_OK;
+	    return 0;
 
 	if (patch->len <= 0)
 	    return XMP_ERR_PATCH;
@@ -609,7 +618,7 @@ int xmp_smix_writepatch(struct xmp_context *ctx, struct patch_info *patch)
 		patch->data);
     }
 
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -619,7 +628,7 @@ int xmp_smix_on(struct xmp_context *ctx)
     int cnt;
 
     if (smix_numbuf)
-	return XMP_OK;
+	return 0;
 
     if (d->numbuf < 1)
 	d->numbuf = 1;
@@ -638,9 +647,7 @@ int xmp_smix_on(struct xmp_context *ctx)
     smix_numvoc = SMIX_NUMVOC;
     extern_drv = TURN_OFF;
 
-    //synth_init (o->freq);
-
-    return XMP_OK;
+    return 0;
 }
 
 
@@ -649,8 +656,6 @@ void xmp_smix_off()
     while (smix_numbuf)
 	free(smix_buffer[--smix_numbuf]);
 
-    //synth_deinit();
-
     free(smix_buf32b);
     free(smix_buffer);
     smix_buf32b = NULL;
@@ -662,15 +667,15 @@ void xmp_smix_off()
 void *xmp_smix_buffer(struct xmp_context *ctx)
 {
     static int outbuf;
-    int act, size;
+    int fmt, size;
     struct xmp_options *o = &ctx->o;
 
     if (!o->resol)
-	act = OUT_U8ULAW;
+	fmt = OUT_U8ULAW;
     else if (o->resol > 8)
-	act = OUT_SU16NORM;
+	fmt = OUT_SU16NORM;
     else
-	act = OUT_SU8NORM;
+	fmt = OUT_SU8NORM;
 
     /* The mixer works with multiple buffers -- this is useless when using
      * multi-buffered sound output (e.g. OSS fragments) but can be useful for
@@ -682,7 +687,7 @@ void *xmp_smix_buffer(struct xmp_context *ctx)
     size = smix_mode * smix_ticksize;
     assert(size <= OUT_MAXLEN);
 
-    out_fn[act](smix_buffer[outbuf], smix_buf32b, size, o->outfmt);
+    out_fn[fmt](smix_buffer[outbuf], smix_buf32b, size, o->amplify, o->outfmt);
 
     smix_resetvar(ctx);
 
