@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996,2001 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: period.c,v 1.10 2007/11/11 12:54:16 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -14,7 +12,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "xmpi.h"
+#include "common.h"
 #include "period.h"
 
 #include <math.h>
@@ -39,13 +37,12 @@ static int period_amiga[] = {
 
 
 /* Get period from note */
-/* Finetune moved to period_to_bend () --claudio */
-inline int note_to_period(int n, int type)
+inline int note_to_period(int n, int f, int type)
 {
-    double d = (double)n;
+    double d = (double)n + (double)f / 128;
 
     return type ?
-	(120 - n) << 4 :			/* Linear */
+	(120.0 - d) * 16 :			/* Linear */
         (int)(6847.0 / pow(2, d / 12));		/* Amiga */
 }
 
@@ -74,7 +71,7 @@ int period_to_note(int p)
 
 
 /* Get pitchbend from base note and period */
-int period_to_bend(int p, int n, int f, int a, int g, int type)
+int period_to_bend(int p, int n, int limit, int gliss, int type)
 {
     int b;
     double d;
@@ -82,7 +79,7 @@ int period_to_bend(int p, int n, int f, int a, int g, int type)
     if (!n)
 	return 0;
 
-    if (a) {				/* Force Amiga limits */
+    if (limit) {				/* Force Amiga limits */
 	if (p > AMIGA_LIMIT_LOWER)
 	    p = AMIGA_LIMIT_LOWER;
 	if (p < AMIGA_LIMIT_UPPER)
@@ -91,17 +88,17 @@ int period_to_bend(int p, int n, int f, int a, int g, int type)
 
     if (type) {
 	/* b = (100 * (((120 - n) << 4) - p)) >> 4 + f * 100 / 128; */
-	b = 100 * (8 * (((120 - n) << 4) - p) + f) / 128;	/* Linear */
-	return g ? b / 100 * 100 : b;
+	b = 100 * (8 * (((120 - n) << 4) - p)) / 128;	/* Linear */
+	return gliss ? b / 100 * 100 : b;
     }
 
     if (p < MIN_PERIOD_A)
 	p = MIN_PERIOD_A;
 
-    d = note_to_period(n, 0);
-    b = 100.0 * ((1536.0 * log(d / p) / M_LN2) + f) / 128;
+    d = note_to_period(n, 0, 0);
+    b = 100.0 * ((1536.0 * log(d / p) / M_LN2)) / 128;
 
-    return g ? b / 100 * 100 : b;	/* Amiga */
+    return gliss ? b / 100 * 100 : b;	/* Amiga */
 }
 
 
@@ -115,6 +112,11 @@ void c2spd_to_note(int c2spd, int *n, int *f)
 {
     int c;
 
+    if (c2spd == 0) {
+	*n = *f = 0;
+	return;
+    }
+
     c = (int)(1536.0 * log((double)c2spd / 8363) / M_LN2);
     *n = c / 128;
     *f = c % 128;
