@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: player.c,v 1.53 2007/12/05 12:24:42 cmatsuoka Exp $
  */
 
 /*
@@ -73,16 +71,16 @@ static int waveform[4][64] = {
 /*
  * "Anyway I think this is the most brilliant piece of crap we
  *  have managed to put up!"
- *                          -- Ice of FC about "Mental Surgery"
+ *			  -- Ice of FC about "Mental Surgery"
  */
 
-static int module_fetch (struct xmp_context *, struct xxm_event *, int, int);
-static void module_play (struct xmp_context *, int, int);
+static int fetch_channel (struct xmp_context *, struct xxm_event *, int, int);
+static void play_channel (struct xmp_context *, int, int);
 
 #include "effects.c"
 
 
-static void dummy()
+static void dummy(unsigned long ul)
 {
     /* dummy */
 }
@@ -150,6 +148,9 @@ static int do_envelope(struct xmp_context *ctx, struct xxm_envinfo *ei, uint16 *
 		*x = env[ei->lps << 1];
     }
 
+    if (chn < 0)
+	return 0;
+
     if (*x > env[rl = (ei->npt - 1) << 1]) {
 	if (!env[rl + 1])
 	    xmp_drv_resetchannel(ctx, chn);
@@ -161,7 +162,7 @@ static int do_envelope(struct xmp_context *ctx, struct xxm_envinfo *ei, uint16 *
 }
 
 
-static inline int chn_copy(struct xmp_player_context *p, int to, int from)
+static inline int copy_channel(struct xmp_player_context *p, int to, int from)
 {
     if (to > 0 && to != from)
 	memcpy(&p->xc_data[to], &p->xc_data[from], sizeof (struct xmp_channel));
@@ -170,7 +171,7 @@ static inline int chn_copy(struct xmp_player_context *p, int to, int from)
 }
 
 
-static inline void chn_reset(struct xmp_context *ctx)
+static inline void reset_channel(struct xmp_context *ctx)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_driver_context *d = &ctx->d;
@@ -188,12 +189,12 @@ static inline void chn_reset(struct xmp_context *ctx)
 	xc = &p->xc_data[i];
 	xc->masterpan = p->m.xxc[i].pan;
 	xc->mastervol = p->m.xxc[i].vol; //0x40;
-        xc->cutoff = 0xff;
+	xc->cutoff = 0xff;
     }
 }
 
 
-static inline void chn_fetch(struct xmp_context *ctx, int ord, int row)
+static inline void fetch_row(struct xmp_context *ctx, int ord, int row)
 {
     int count, chn;
     struct xmp_player_context *p = &ctx->p;
@@ -201,7 +202,7 @@ static inline void chn_fetch(struct xmp_context *ctx, int ord, int row)
 
     count = 0;
     for (chn = 0; chn < p->m.xxh->chn; chn++) {
-	if (module_fetch(ctx, &EVENT(ord, chn, row), chn, 1) != XMP_OK) {
+	if (fetch_channel(ctx, &EVENT(ord, chn, row), chn, 1) != 0) {
 	    p->fetch_ctl[chn]++;
 	    count++;
 	}
@@ -209,7 +210,7 @@ static inline void chn_fetch(struct xmp_context *ctx, int ord, int row)
 
     for (chn = 0; count; chn++) {
 	if (p->fetch_ctl[chn]) {
-	    module_fetch(ctx, &EVENT(ord, chn, row), chn, 0);
+	    fetch_channel(ctx, &EVENT(ord, chn, row), chn, 0);
 	    p->fetch_ctl[chn] = 0;
 	    count--;
 	}
@@ -217,17 +218,17 @@ static inline void chn_fetch(struct xmp_context *ctx, int ord, int row)
 }
 
 
-static inline void chn_refresh(struct xmp_context *ctx, int tick)
+static inline void play_frame(struct xmp_context *ctx, int tick)
 { 
     struct xmp_driver_context *d = &ctx->d;
     int i;
 
-    for (i = d->numchn; i--;)
-	module_play(ctx, i, tick);
+    for (i = 0; i < d->numchn; i++)
+	play_channel(ctx, i, tick);
 }
 
 
-static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, int ctl)
+static int fetch_channel(struct xmp_context *ctx, struct xxm_event *e, int chn, int ctl)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_mod_context *m = &p->m;
@@ -235,13 +236,50 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
     struct xmp_channel *xc;
     int cont_sample;
 
+    xc = &p->xc_data[chn];
+
+    /* Tempo affects delay and must be computed first */
+    if ((e->fxt == FX_TEMPO && e->fxp < 0x20) || e->fxt == FX_S3M_TEMPO) {
+	if (e->fxp)
+	    p->tempo = e->fxp;
+    }
+    if ((e->f2t == FX_TEMPO && e->f2p < 0x20) || e->f2t == FX_S3M_TEMPO) {
+	if (e->f2p)
+	    p->tempo = e->f2p;
+    }
+
+    /* Delay the entire fetch cycle */
+    if (e->fxt == FX_EXTENDED && MSN(e->fxp) == EX_DELAY) {
+	xc->delay = LSN(e->fxp) + 1;
+	xc->delayed_event = e;
+	if (e->ins)
+		xc->delayed_ins = e->ins;
+	e->fxt = e->fxp = 0;
+	return 0;
+    }
+    if (e->f2t == FX_EXTENDED && MSN(e->f2p) == EX_DELAY) {
+	xc->delay = LSN(e->f2p) + 1;
+	xc->delayed_event = e;
+	if (e->ins)
+		xc->delayed_ins = e->ins;
+	e->f2t = e->f2p = 0;
+	return 0;
+    }
+
+    /* Emulate Impulse Tracker "always read instrument" bug */
+    if (e->note && !e->ins && xc->delayed_ins && m->fetch & XMP_CTL_SAVEINS) {
+	e->ins = xc->delayed_ins;
+	xc->delayed_ins = 0;
+    }
+
     flg = 0;
     smp = ins = note = -1;
-    xc = &p->xc_data[chn];
     xins = xc->ins;
     key = e->note;
     cont_sample = 0;
 
+    /* Check instrument */
+
     if (e->ins) {
 	ins = e->ins - 1;
 	flg = NEW_INS | RESET_VOL | RESET_ENV;
@@ -275,16 +313,18 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 	xc->med_arp = xc->med_aidx = 0;
     }
 
+    /* Check note */
+
     if (key) {
 	flg |= NEW_NOTE;
 
-        if (key == XMP_KEY_FADE) {
-            SET(FADEOUT);
+	if (key == XMP_KEY_FADE) {
+	    SET(FADEOUT);
 	    flg &= ~(RESET_VOL | RESET_ENV);
-        } else if (key == XMP_KEY_CUT) {
-            xmp_drv_resetchannel(ctx, chn);
-        } else if (key == XMP_KEY_OFF) {
-            SET(RELEASE);
+	} else if (key == XMP_KEY_CUT) {
+	    xmp_drv_resetchannel(ctx, chn);
+	} else if (key == XMP_KEY_OFF) {
+	    SET(RELEASE);
 	    flg &= ~(RESET_VOL | RESET_ENV);
 	} else if (e->fxt == FX_TONEPORTA || e->f2t == FX_TONEPORTA) {
 	    /* This test should fix portamento behaviour in 7spirits.s3m */
@@ -307,18 +347,18 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 		/* set key to 0 so we can have the tone portamento from
 		 * the original note (see funky_stars.xm pos 5 ch 9)
 		 */
-        	key = 0;
+		key = 0;
 
 		/* And do the same if there's no keyoff (see comic bakery
 		 * remix.xm pos 1 ch 3)
 		 */
 	    }
-        } else if (flg & NEW_INS) {
-            xins = ins;
-        } else {
-            ins = xc->insdef;
-            flg |= IS_READY;
-        }
+	} else if (flg & NEW_INS) {
+	    xins = ins;
+	} else {
+	    ins = xc->insdef;
+	    flg |= IS_READY;
+	}
     }
 
     if (!key || key >= XMP_KEY_OFF)
@@ -327,10 +367,10 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
     if ((uint32)ins < m->xxh->ins && m->xxih[ins].nsm)
 	flg |= IS_VALID;
 
-    if ((uint32)key < XMP_KEY_OFF && key--) {
-	xc->key = key;
+    if ((uint32)key < XMP_KEY_OFF && key > 0) {
+	xc->key = --key;
 
-        if (flg & IS_VALID) {
+	if (flg & IS_VALID && key < XXM_KEY_MAX) {
 	    if (m->xxim[ins].ins[key] != 0xff) {
 		note = key + m->xxi[ins][m->xxim[ins].ins[key]].xpo +
 						m->xxim[ins].xpo[key];
@@ -345,7 +385,7 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
     }
 
     if (smp >= 0) {
-	if (chn_copy(p, xmp_drv_setpatch(ctx, chn, ins, smp, note,
+	if (copy_channel(p, xmp_drv_setpatch(ctx, chn, ins, smp, note,
 	 		m->xxi[ins][m->xxim[ins].ins[key]].nna,
 	   		m->xxi[ins][m->xxim[ins].ins[key]].dct,
 			m->xxi[ins][m->xxim[ins].ins[key]].dca, ctl,
@@ -358,16 +398,18 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 
     /* Reset flags */
     xc->delay = xc->retrig = 0;
-    xc->flags = flg | (xc->flags & 0xff000000);
+    xc->flags = flg | (xc->flags & 0xff000000);	/* keep persistent flags */
 
     xc->a_idx = 0;
     xc->a_size = 1;
     xc->a_val[0] = 0;
 
-    if ((uint32)xins >= m->xxh->ins || !m->xxih[xins].nsm)
+    if ((uint32)xins >= m->xxh->ins || !m->xxih[xins].nsm) {
 	RESET(IS_VALID);
-    else
+    } else {
 	SET(IS_VALID);
+    }
+
     xc->ins = xins;
 
     /* Process new volume */
@@ -388,7 +430,7 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 
     if (!TEST(IS_VALID)) {
 	xc->volume = 0;
-	return XMP_OK;
+	return 0;
     }
 
     if (note >= 0) {
@@ -397,7 +439,7 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 	if (cont_sample == 0) {
 	    xmp_drv_voicepos(ctx, chn, xc->offset_val);
 	    if (TEST(OFFSET) && (m->fetch & XMP_CTL_FX9BUG))
-	        xc->offset_val <<= 1;
+		xc->offset_val <<= 1;
 	}
 	RESET(OFFSET);
 
@@ -408,40 +450,41 @@ static int module_fetch(struct xmp_context *ctx, struct xxm_event *e, int chn, i
 	if (!TEST(FINETUNE))
 	    xc->finetune = m->xxi[ins][m->xxim[ins].ins[key]].fin;
 
-	xc->s_end = xc->period = note_to_period(note, m->xxh->flg & XXM_FLG_LINEAR);
+	xc->s_end = xc->period = note_to_period(note, xc->finetune,
+			m->xxh->flg & XXM_FLG_LINEAR);
 
-	xc->y_idx = xc->t_idx = 0;              /* #?# Onde eu coloco isso? */
+	xc->y_idx = xc->t_idx = 0;	      /* #?# Onde eu coloco isso? */
 
 	SET(ECHOBACK);
     }
 
     if (xc->key == 0xff || XXIM.ins[xc->key] == 0xff)
-        return XMP_OK;
+	return 0;
 
     if (TEST(RESET_ENV)) {
-        /* xc->fadeout = 0x8000; -- moved to fetch */
-        RESET(RELEASE | FADEOUT);
-        xc->gvl = XXI[XXIM.ins[xc->key]].gvl;   /* #?# Onde eu coloco isso? */
-        xc->insvib_swp = XXI->vsw;              /* #?# Onde eu coloco isso? */
-        xc->insvib_idx = 0;                     /* #?# Onde eu coloco isso? */
-        xc->v_idx = xc->p_idx = xc->f_idx = 0;
+	/* xc->fadeout = 0x8000; -- moved to fetch */
+	RESET(RELEASE | FADEOUT);
+	xc->gvl = XXI[XXIM.ins[xc->key]].gvl;   /* #?# Onde eu coloco isso? */
+	xc->insvib_swp = XXI->vsw;	      /* #?# Onde eu coloco isso? */
+	xc->insvib_idx = 0;		     /* #?# Onde eu coloco isso? */
+	xc->v_idx = xc->p_idx = xc->f_idx = 0;
 	xc->cutoff = XXI->ifc & 0x80 ? (XXI->ifc - 0x80) * 2 : 0xff;
 	xc->resonance = XXI->ifr & 0x80 ? (XXI->ifr - 0x80) * 2 : 0;
     }
 
     if (TEST(RESET_VOL)) {
-        xc->volume = XXI[XXIM.ins[xc->key]].vol;
-        SET(ECHOBACK | NEW_VOL);
+	xc->volume = XXI[XXIM.ins[xc->key]].vol;
+	SET(ECHOBACK | NEW_VOL);
     }
 
-    if ((m->fetch & XMP_CTL_ST3GVOL) && TEST(NEW_VOL))
+    if ((m->fetch & XMP_CTL_ST3GVOL) && TEST(NEW_VOL) && m->volbase)
 	xc->volume = xc->volume * m->volume / m->volbase;
 
-    return XMP_OK;
+    return 0;
 }
 
 
-static void module_play(struct xmp_context *ctx, int chn, int t)
+static void play_channel(struct xmp_context *ctx, int chn, int t)
 {
     struct xmp_channel *xc;
     int finalvol, finalpan, cutoff, act;
@@ -453,11 +496,17 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
     struct xmp_mod_context *m = &p->m;
     struct xmp_options *o = &ctx->o;
 
+    xc = &p->xc_data[chn];
+
+    /* Do delay */
+    if (xc->delay && !--xc->delay) {
+	if (fetch_channel(ctx, xc->delayed_event, chn, 1) != 0)
+	    fetch_channel(ctx, xc->delayed_event, chn, 0);
+    }
+
     if ((act = xmp_drv_cstat(ctx, chn)) == XMP_CHN_DUMB)
 	return;
 
-    xc = &p->xc_data[chn];
-
     if (!t && act != XMP_CHN_ACTIVE) {
 	if (!TEST(IS_VALID) || act == XMP_ACT_CUT) {
 	    xmp_drv_resetchannel(ctx, chn);
@@ -491,10 +540,10 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 	     * can release it.
 	     */
 	     if (m->fetch & XMP_CTL_VIRTUAL) {
-	         xmp_drv_resetchannel(ctx, chn);
-	         return;
+		 xmp_drv_resetchannel(ctx, chn);
+		 return;
 	     } else {
-	         xc->volume = 0;
+		 xc->volume = 0;
 	     }
 	}
     }
@@ -506,7 +555,7 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 	get_envelope((int16 *)XXPE, XXIH.pei.npt, xc->p_idx) : 32;
 
     frq_envelope = XXIH.fei.flg & XXM_ENV_ON ?
-        (int16)get_envelope((int16 *)XXFE, XXIH.fei.npt, xc->f_idx) : 0;
+	(int16)get_envelope((int16 *)XXFE, XXIH.fei.npt, xc->f_idx) : 0;
 
     /* Update envelopes */
     if (do_envelope(ctx, &XXIH.aei, XXAE, &xc->v_idx, DOENV_RELEASE, chn))
@@ -518,7 +567,8 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
     if (TEST(NOTE_SLIDE)) {
 	if (!--xc->ns_count) {
 	    xc->note += xc->ns_val;
-	    xc->period = note_to_period(xc->note, m->xxh->flg & XXM_FLG_LINEAR);
+	    xc->period = note_to_period(xc->note, xc->finetune,
+				m->xxh->flg & XXM_FLG_LINEAR);
 	    xc->ns_count = xc->ns_speed;
 	}
     }
@@ -570,7 +620,7 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
     xc->pitchbend = period_to_bend(
 	xc->period + vibrato + med_vibrato,
 	xc->note,
-	xc->finetune,
+	/* xc->finetune, */
 	m->xxh->flg & XXM_FLG_MODRNG,
 	xc->gliss,
 	m->xxh->flg & XXM_FLG_LINEAR);
@@ -586,13 +636,17 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
      *       it may be a bug.
      */
 
-    finalpan = m->fetch & XMP_CTL_DYNPAN ?
-	xc->pan + (pan_envelope - 32) * (128 - abs (xc->pan - 128)) / 32 : 0x80;
+    finalpan = m->fetch & XMP_CTL_DYNPAN ?  xc->pan + (pan_envelope - 32) *
+			(128 - abs (xc->pan - 128)) / 32 : 0x80;
     finalpan = xc->masterpan + (finalpan - 128) *
-	(128 - abs (xc->masterpan - 128)) / 128;
+			(128 - abs (xc->masterpan - 128)) / 128;
 
-    cutoff = XXIH.fei.flg & XXM_ENV_FLT ? frq_envelope : 0xff;
-    cutoff = xc->cutoff * cutoff / 0xff;
+    if (o->cf_cutoff) {
+	cutoff = o->cf_cutoff;		/* Click-filter cutoff */
+    } else {
+	cutoff = XXIH.fei.flg & XXM_ENV_FLT ? frq_envelope : 0xff;
+	cutoff = xc->cutoff * cutoff / 0xff;
+    }
 
     /* Echoback events */
     if (chn < d->numtrk) {
@@ -608,22 +662,27 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 	}
     }
 
+#if 0
     /* Do delay */
     if (xc->delay) {
-	if (--xc->delay)
+	if (--xc->delay) {
 	    finalvol = 0;
-	else
+	} else {
 	    xmp_drv_retrig(ctx, chn);
+	}
     }
+#endif
 
     /* Do tremor */
-    if (xc->tremor) {
-	if (xc->tcnt_dn > 0)
+    if (xc->tcnt_up || xc->tcnt_dn) {
+	if (xc->tcnt_up > 0) {
+	    if (xc->tcnt_up--)
+		xc->tcnt_dn = LSN(xc->tremor);
+	} else {
 	    finalvol = 0;
-	if (!xc->tcnt_up--)
-	    xc->tcnt_dn = LSN (xc->tremor);
-	if (!xc->tcnt_dn--)
-	    xc->tcnt_up = MSN (xc->tremor);
+	    if (xc->tcnt_dn--)
+		xc->tcnt_up = MSN(xc->tremor);
+	}
     }
 
     /* Do keyoff */
@@ -680,6 +739,16 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 	if (TEST(PITCHBEND) || TEST_PER(PITCHBEND))
 	    xc->period += xc->f_val;
 
+	/* Do tone portamento */
+	if (TEST(TONEPORTA) || TEST_PER(TONEPORTA)) {
+	    xc->period += xc->s_sgn * xc->s_val;
+	    if ((xc->s_sgn * xc->s_end) < (xc->s_sgn * xc->period)) {
+		xc->period = xc->s_end;
+		RESET(TONEPORTA);
+		RESET_PER(TONEPORTA);
+   	    }
+	}
+
 	/* Workaround for panic.s3m (from Toru Egashira's NSPmod) */
 	if (xc->period <= 8)
 	    xc->volume = 0;
@@ -688,14 +757,15 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
     if (t % p->tempo == 0) {
 	/* Process "fine" effects */
 	if (TEST(FINE_VOLS))
-            xc->volume += xc->v_fval;
+	    xc->volume += xc->v_fval;
 	if (TEST(FINE_BEND))
 	    xc->period = (4 * xc->period + xc->f_fval) / 4;
 	if (TEST(TRK_FVSLIDE))
-            xc->mastervol += xc->trk_fval;
+	    xc->mastervol += xc->trk_fval;
 	if (TEST(FINE_NSLIDE)) {
-            xc->note += xc->ns_fval;
-	    xc->period = note_to_period(xc->note, m->xxh->flg & XXM_FLG_LINEAR);
+	    xc->note += xc->ns_fval;
+	    xc->period = note_to_period(xc->note, xc->finetune,
+				m->xxh->flg & XXM_FLG_LINEAR);
 	}
     }
 
@@ -721,16 +791,6 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 	    xc->period = MAX_PERIOD_A;
     }
 
-    /* Do tone portamento */
-    if (TEST(TONEPORTA) || TEST_PER(TONEPORTA)) {
-	xc->period += xc->s_sgn * xc->s_val;
-	if ((xc->s_sgn * xc->s_end) < (xc->s_sgn * xc->period)) {
-	    xc->period = xc->s_end;
-	    RESET(TONEPORTA);
-	    RESET_PER(TONEPORTA);
-	}
-    }
-
     /* Update vibrato, tremolo and arpeggio indexes */
     xc->insvib_idx += XXI->vra >> 2;
     xc->insvib_idx %= 64;
@@ -748,7 +808,7 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
     /* Process MED synth arpeggio */
     med_arp = get_med_arp(p, xc);
 
-    /* Adjust pitch and pan, than play the note */
+    /* Adjust pitch and pan, then play the note */
     finalpan = o->outfmt & XMP_FMT_MONO ?
 	0 : (finalpan - 0x80) * o->mix / 100;
     xmp_drv_setbend(ctx, chn, xc->pitchbend + xc->a_val[xc->a_idx] + med_arp);
@@ -771,17 +831,21 @@ static void module_play(struct xmp_context *ctx, int chn, int t)
 }
 
 
-int xmpi_player_start(struct xmp_context *ctx)
+int _xmp_player_start(struct xmp_context *ctx)
 {
-    int r, t, e, ord;		/* rows, tick, end point, order */
+    int num_rows, frame, e, ord;	/* rows, frame, end point, order */
+    int ret;
     double playing_time;
     struct xmp_player_context *p = &ctx->p;
     struct xmp_driver_context *d = &ctx->d;
     struct xmp_mod_context *m = &p->m;
     struct xmp_options *o = &ctx->o;
+    struct flow_control *f = &p->flow;
 
-    if (m->xxh->len == 0 || m->xxh->chn == 0)
-	return XMP_OK;
+    if (m->xxh->len == 0 || m->xxh->chn == 0) {
+	xmp_drv_writepatch(ctx, NULL);
+	return 0;
+    }
 
     if (xmp_event_callback == NULL)
 	xmp_event_callback = dummy;
@@ -796,23 +860,23 @@ int xmpi_player_start(struct xmp_context *ctx)
 
     m->volume = m->xxo_info[ord].gvl;
     p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[ord].bpm);
-    p->flow.jumpline = m->xxo_fstrow[ord];
+    f->jumpline = m->xxo_fstrow[ord];
     p->tempo = m->xxo_info[ord].tempo;
     playing_time = 0;
 
-    if ((r = xmp_drv_on(ctx, m->xxh->chn)) != XMP_OK)
-	return r;
+    if ((ret = xmp_drv_on(ctx, m->xxh->chn)) != 0)
+	return ret;
 
-    p->flow.jump = -1;
+    f->jump = -1;
 
     p->fetch_ctl = calloc(m->xxh->chn, sizeof (int));
-    p->flow.loop_stack = calloc(d->numchn, sizeof (int));
-    p->flow.loop_row = calloc(d->numchn, sizeof (int));
+    f->loop_stack = calloc(d->numchn, sizeof (int));
+    f->loop_start = calloc(d->numchn, sizeof (int));
     p->xc_data = calloc(d->numchn, sizeof (struct xmp_channel));
-    if (!(p->fetch_ctl && p->flow.loop_stack && p->flow.loop_row && p->xc_data))
+    if (!(p->fetch_ctl && f->loop_stack && f->loop_start && p->xc_data))
 	return XMP_ERR_ALLOC;
 
-    chn_reset(ctx);
+    reset_channel(ctx);
 
     xmp_drv_starttimer(ctx);
 
@@ -820,24 +884,24 @@ int xmpi_player_start(struct xmp_context *ctx)
 next_order:
 	if (ord >= m->xxh->len) {
 	    ord = ((uint32)m->xxh->rst > m->xxh->len ||
-		(uint32)m->xxo[m->xxh->rst] >= m->xxh->pat) ?  0 : m->xxh->rst;
+		(uint32)m->xxo[m->xxh->rst] >= m->xxh->pat) ? 0 : m->xxh->rst;
 	    m->volume = m->xxo_info[ord].gvl;
 	}
 
 	/* Skip invalid patterns */
 	if (m->xxo[ord] >= m->xxh->pat) {
 #if 0
-           if (m->xxo[ord] == 0xff)            /* S3M uses 0xff as end mark */
-                ord = m->xxh->len;
+	   if (m->xxo[ord] == 0xff)	    /* S3M uses 0xff as end mark */
+		ord = m->xxh->len;
 #endif
 	    continue;
 	}
 
-	r = m->xxp[m->xxo[ord]]->rows;
-	if (p->flow.jumpline >= r)
-	    p->flow.jumpline = 0;
-	p->flow.row_cnt = p->flow.jumpline;
-	p->flow.jumpline = 0;
+	num_rows = m->xxp[m->xxo[ord]]->rows;
+	if (f->jumpline >= num_rows)
+	    f->jumpline = 0;
+	f->row = f->jumpline;
+	f->jumpline = 0;
 
 	p->pos = ord;
 
@@ -848,9 +912,12 @@ next_order:
 		p->xc_data[chn].per_flags = 0;
 	}
 
-	for (; p->flow.row_cnt < r; p->flow.row_cnt++) {
+	/* Row processing */
+	for (; f->row < num_rows; f->row++) {
+
+	    /* Check module end */
 	    if ((~m->fetch & XMP_CTL_LOOP) && ord == p->xmp_scan_ord &&
-		p->flow.row_cnt == p->xmp_scan_row) {
+		f->row == p->xmp_scan_row) {
 		if (!e--)
 		    goto end_module;
 	    }
@@ -858,15 +925,16 @@ next_order:
 	    if (p->pause) {
 		xmp_drv_stoptimer(ctx);
 		while (p->pause) {
-		    xmpi_select_read(1, 125);
+		    usleep(125000);
 		    xmp_event_callback(0);
 		}
 		xmp_drv_starttimer(ctx);
 	    }
 
-	    for (t = 0; t < (p->tempo * (1 + p->flow.delay)); t++) {
-		/* xmp_player_ctl processing */
+	    /* Frame processing */
+	    for (frame = 0; frame < (p->tempo * (1 + f->delay)); frame++) {
 
+		/* Check reposition */
 		if (ord != p->pos) {
 		    if (p->pos == -1)
 			p->pos++;		/* restart module */
@@ -882,31 +950,34 @@ next_order:
 		    p->tempo = m->xxo_info[ord = p->pos].tempo;
 		    p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[ord].bpm);
 		    m->volume = m->xxo_info[ord].gvl;
-		    p->flow.jump = ord;
-		    p->flow.jumpline = m->xxo_fstrow[ord--];
-		    p->flow.row_cnt = -1;
+		    f->jump = ord;
+		    f->jumpline = m->xxo_fstrow[ord--];
+		    f->row = -1;
 		    xmp_drv_bufwipe(ctx);
 		    xmp_drv_sync(ctx, 0);
 		    xmp_drv_reset(ctx);
-		    chn_reset(ctx);
+		    reset_channel(ctx);
 		    break;
 		}
 
-		if (!t) {
+		if (frame == 0) {		/* first frame of row */
 		    p->gvol_flag = 0;
-		    chn_fetch(ctx, m->xxo[ord], p->flow.row_cnt);
 
-		    xmp_drv_echoback(ctx, (p->tempo << 12) | (p->xmp_bpm << 4) |
-							XMP_ECHO_BPM);
+		    fetch_row(ctx, m->xxo[ord], f->row);
+
+		    xmp_drv_echoback(ctx, (p->tempo << 12) |
+					(p->xmp_bpm << 4) | XMP_ECHO_BPM);
 		    xmp_drv_echoback(ctx, (m->volume << 4) | XMP_ECHO_GVL);
-		    xmp_drv_echoback(ctx, (m->xxo[ord] << 12) | (ord << 4) |
-							XMP_ECHO_ORD);
+		    xmp_drv_echoback(ctx, (m->xxo[ord] << 12) |
+					(ord << 4) | XMP_ECHO_ORD);
 		    xmp_drv_echoback(ctx, (d->numvoc << 4) | XMP_ECHO_NCH);
 		    xmp_drv_echoback(ctx, ((m->xxp[m->xxo[ord]]->rows - 1)
-				<< 12) | (p->flow.row_cnt << 4) | XMP_ECHO_ROW);
+				<< 12) | (f->row << 4) | XMP_ECHO_ROW);
 		}
-		xmp_drv_echoback(ctx, (t << 4) | XMP_ECHO_FRM);
-		chn_refresh(ctx, t);
+
+		xmp_drv_echoback(ctx, (frame << 4) | XMP_ECHO_FRM);
+
+		play_frame(ctx, frame);
 
 		if (o->time && (o->time < playing_time))
 		    goto end_module;
@@ -922,25 +993,25 @@ next_order:
 		xmp_drv_bufdump(ctx);
 	    }
 
-	    p->flow.delay = 0;
+	    f->delay = 0;
 
-	    if (p->flow.row_cnt == -1)
+	    if (f->row == -1)
 		break;
 
-	    if (p->flow.pbreak) {
-		p->flow.pbreak = 0;
+	    if (f->pbreak) {
+		f->pbreak = 0;
 		break;
 	    }
 
-	    if (p->flow.loop_chn) {
-		p->flow.row_cnt = p->flow.loop_row[--p->flow.loop_chn] - 1;
-		p->flow.loop_chn = 0;
+	    if (f->loop_chn) {
+		f->row = f->loop_start[--f->loop_chn] - 1;
+		f->loop_chn = 0;
 	    }
 	}
 
-	if (p->flow.jump != -1) {
-	    ord = p->flow.jump;
-	    p->flow.jump = -1;
+	if (f->jump != -1) {
+	    ord = f->jump;
+	    f->jump = -1;
 	    goto next_order;
 	}
     }
@@ -952,11 +1023,247 @@ end_module:
     xmp_drv_stoptimer(ctx);
 
     free(p->xc_data);
-    free(p->flow.loop_row);
-    free(p->flow.loop_stack);
+    free(f->loop_start);
+    free(f->loop_stack);
     free(p->fetch_ctl);
 
     xmp_drv_off(ctx);
 
-    return XMP_OK;
+    return 0;
 }
+
+
+#ifdef TEST_OPEN_LOOP
+/*
+ * EXPERIMENTAL open loop
+ */
+
+int xmp_player_start(struct xmp_context *ctx)
+{
+    //int r, t, e;		/* rows, tick, end point, order */
+	int ret;
+	struct xmp_player_context *p = &ctx->p;
+	struct xmp_driver_context *d = &ctx->d;
+	struct xmp_mod_context *m = &p->m;
+	struct xmp_options *o = &ctx->o;
+	struct flow_control *f = &p->flow;
+
+	if (m->xxh->len == 0 || m->xxh->chn == 0)
+		return 0;
+
+	if (xmp_event_callback == NULL)
+		xmp_event_callback = dummy;
+
+	p->gvol_slide = 0;
+	p->gvol_base = m->volbase;
+	f->ord = o->start;
+	f->frame = 0;
+	f->row = 0;
+	f->num_rows = m->xxp[m->xxo[f->ord]]->rows;
+
+	/* Skip invalid patterns at start (the seventh laboratory.it) */
+	while (f->ord < m->xxh->len && m->xxo[f->ord] >= m->xxh->pat)
+		f->ord++;
+
+	m->volume = m->xxo_info[f->ord].gvl;
+	p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[f->ord].bpm);
+	p->tempo = m->xxo_info[f->ord].tempo;
+	f->jumpline = m->xxo_fstrow[f->ord];
+	f->playing_time = 0;
+	f->end_point = p->xmp_scan_num;
+
+	if ((ret = xmp_drv_on(ctx, m->xxh->chn)) != 0)
+		return ret;
+
+	f->jump = -1;
+
+	p->fetch_ctl = calloc(m->xxh->chn, sizeof (int));
+	f->loop_stack = calloc(d->numchn, sizeof (int));
+	f->loop_start = calloc(d->numchn, sizeof (int));
+	p->xc_data = calloc(d->numchn, sizeof (struct xmp_channel));
+	if (!(p->fetch_ctl && f->loop_stack && f->loop_start && p->xc_data))
+		return XMP_ERR_ALLOC;
+
+	reset_channel(ctx);
+
+	xmp_drv_starttimer(ctx);
+
+	return 0;
+}
+
+
+int xmp_player_loop(struct xmp_context *ctx)
+{
+	struct xmp_player_context *p = &ctx->p;
+	struct xmp_driver_context *d = &ctx->d;
+	struct xmp_mod_context *m = &p->m;
+	struct xmp_options *o = &ctx->o;
+	struct flow_control *f = &p->flow;
+
+	if (f->frame == 0) {			/* first frame in row */
+		/* check end of module */
+	    	if ((~m->fetch & XMP_CTL_LOOP) && f->ord == p->xmp_scan_ord &&
+					f->row == p->xmp_scan_row) {
+			if (!f->end_point--)
+				return -1;
+		}
+
+		p->gvol_flag = 0;
+		fetch_row(ctx, m->xxo[f->ord], f->row);
+
+		xmp_drv_echoback(ctx, (p->tempo << 12) | (p->xmp_bpm << 4) |
+							XMP_ECHO_BPM);
+		xmp_drv_echoback(ctx, (m->volume << 4) | XMP_ECHO_GVL);
+		xmp_drv_echoback(ctx, (m->xxo[f->ord] << 12) | (f->ord << 4) |
+							XMP_ECHO_ORD);
+		xmp_drv_echoback(ctx, (d->numvoc << 4) | XMP_ECHO_NCH);
+		xmp_drv_echoback(ctx, ((m->xxp[m->xxo[f->ord]]->rows - 1)
+				<< 12) | (f->row << 4) | XMP_ECHO_ROW);
+	}
+
+	/* For each frame */
+
+	if (f->ord != p->pos) {			/* changed pattern */
+		if (p->pos == -1)
+			p->pos++;		/* restart module */
+
+		if (p->pos == -2) {		/* set by xmp_module_stop */
+			xmp_drv_bufwipe(ctx);
+			return -1;		/* that's all folks */
+		}
+
+		if (p->pos == 0)
+			f->end_point = p->xmp_scan_num;
+
+		p->tempo = m->xxo_info[f->ord = p->pos].tempo;
+		p->tick_time = m->rrate / (p->xmp_bpm = m->xxo_info[f->ord].bpm);
+		m->volume = m->xxo_info[f->ord].gvl;
+		f->jump = f->ord;
+		f->jumpline = m->xxo_fstrow[f->ord--];
+		f->row = -1;
+		xmp_drv_bufwipe(ctx);
+		xmp_drv_sync(ctx, 0);
+		xmp_drv_reset(ctx);
+		reset_channel(ctx);
+		goto next_row;
+	}
+
+
+	xmp_drv_echoback(ctx, (f->frame << 4) | XMP_ECHO_FRM);
+	play_frame(ctx, f->frame);
+
+	if (o->time && (o->time < f->playing_time))	/* expired time */
+		return -1;
+
+	if (m->fetch & XMP_CTL_MEDBPM) {
+		xmp_drv_sync(ctx, p->tick_time * 33 / 125);
+		f->playing_time += m->rrate * 33 / (100 * p->xmp_bpm * 125);
+	} else {
+		xmp_drv_sync(ctx, p->tick_time);
+		f->playing_time += m->rrate / (100 * p->xmp_bpm);
+	}
+
+	xmp_drv_bufdump(ctx);
+
+
+
+	/* check new row */
+	if (f->frame >= (p->tempo * (1 + f->delay)) - 1) {
+next_row:
+//printf("** new row\n");
+		f->frame = 0;
+		f->delay = 0;
+
+		if (f->row == -1)
+			goto next_row;
+
+		if (f->pbreak) {
+			f->pbreak = 0;
+			goto next_row;
+		}
+
+		if (f->loop_chn) {
+			f->row = f->loop_start[--f->loop_chn] - 1;
+			f->loop_chn = 0;
+		}
+
+		f->row++;
+
+		if (f->jump != -1) {
+			f->ord = f->jump;
+			f->jump = -1;
+			goto next_order;
+		}
+
+#if 0
+		if (p->pause) {
+			xmp_drv_stoptimer(ctx);
+			while (p->pause) {
+				usleep(125000);
+				xmp_event_callback(0);
+			}
+			xmp_drv_starttimer(ctx);
+	    	}
+#endif
+
+
+		/* check end of pattern */
+		if (f->row >= f->num_rows) {
+next_order:
+//printf("*** new order\n");
+    			f->ord++;
+
+			/* Restart module */
+			if (f->ord >= m->xxh->len) {
+    				f->ord = ((uint32)m->xxh->rst > m->xxh->len ||
+					(uint32)m->xxo[m->xxh->rst] >=
+					m->xxh->pat) ?  0 : m->xxh->rst;
+				m->volume = m->xxo_info[f->ord].gvl;
+			}
+
+			/* Skip invalid patterns */
+			if (m->xxo[f->ord] >= m->xxh->pat) {
+    				f->ord++;
+    				goto next_order;
+			}
+
+			f->num_rows = m->xxp[m->xxo[f->ord]]->rows;
+			if (f->jumpline >= f->num_rows)
+				f->jumpline = 0;
+			f->row = f->jumpline;
+			f->jumpline = 0;
+
+			p->pos = f->ord;
+
+			/* Reset persistent effects at new pattern */
+			if (m->fetch & XMP_CTL_PERPAT) {
+				int chn;
+				for (chn = 0; chn < p->m.xxh->chn; chn++)
+					p->xc_data[chn].per_flags = 0;
+			}
+		}
+	} else {
+		f->frame++;
+	}
+
+	return 0;
+}
+    
+
+void xmp_player_end(struct xmp_context *ctx)
+{
+	struct xmp_player_context *p = &ctx->p;
+
+	xmp_drv_echoback(ctx, XMP_ECHO_END);
+	while (xmp_drv_getmsg(ctx) != XMP_ECHO_END)
+		xmp_drv_bufdump(ctx);
+	xmp_drv_stoptimer(ctx);
+
+	free(p->xc_data);
+	free(f->loop_start);
+	free(f->loop_stack);
+	free(p->fetch_ctl);
+
+	xmp_drv_off(ctx);
+}
+#endif
