@@ -1,11 +1,9 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
  * for more information.
- *
- * $Id: readrc.c,v 1.16 2007/12/02 01:19:45 cmatsuoka Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,8 +13,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <sys/param.h>
-#include "xmpi.h"
+#include <limits.h>
+#include "common.h"
 
 #ifdef __AMIGA__
 #include <sys/unistd.h>
@@ -46,20 +44,19 @@ static int get_yesno(char *s)
 }
 
 
-int xmpi_read_rc(struct xmp_context *ctx)
+int _xmp_read_rc(struct xmp_context *ctx)
 {
     struct xmp_options *o = &ctx->o;
     FILE *rc;
-    char myrc[MAXPATHLEN];
+    char myrc[PATH_MAX];
     char *hash, *var, *val, line[256];
     char cparm[512];
 
 #if defined __EMX__
-    char myrc2[MAXPATHLEN];
+    char myrc2[PATH_MAX];
     char *home = getenv("HOME");
 
-    snprintf(myrc, MAXPATHLEN, "%s\\.xmp\\xmp.conf", home);
-    snprintf(myrc2, MAXPATHLEN, "%s\\.xmprc", home);    
+    snprintf(myrc, PATH_MAX, "%s\\.xmp\\xmp.conf", home);
 
     if ((rc = fopen(myrc2, "r")) == NULL) {
 	if ((rc = fopen(myrc, "r")) == NULL) {
@@ -69,22 +66,26 @@ int xmpi_read_rc(struct xmp_context *ctx)
 	}
     }
 #elif defined __AMIGA__
-    strncpy(myrc, "PROGDIR:xmp.conf", MAXPATHLEN);
+    strncpy(myrc, "PROGDIR:xmp.conf", PATH_MAX);
+
+    if ((rc = fopen(myrc, "r")) == NULL)
+	return -1;
+#elif defined WIN32
+    char *home = getenv("SystemRoot");
+
+    snprintf(myrc, PATH_MAX, "%s/xmp.ini", home);
 
     if ((rc = fopen(myrc, "r")) == NULL)
 	return -1;
 #else
-    char myrc2[MAXPATHLEN];
     char *home = getenv("HOME");
 
-    snprintf(myrc, MAXPATHLEN, "%s/.xmp/xmp.conf", home);
-    snprintf(myrc2, MAXPATHLEN, "%s/.xmprc", home);
+    snprintf(myrc, PATH_MAX, "%s/.xmp/xmp.conf", home);
+    /*snprintf(myrc2, PATH_MAX, "%s/.xmprc", home); -- deprecated */
 
-    if ((rc = fopen(myrc2, "r")) == NULL) {
-	if ((rc = fopen(myrc, "r")) == NULL) {
-	    if ((rc = fopen(SYSCONFDIR "/xmp.conf", "r")) == NULL) {
-		return -1;
-	    }
+    if ((rc = fopen(myrc, "r")) == NULL) {
+	if ((rc = fopen(SYSCONFDIR "/xmp.conf", "r")) == NULL) {
+	    return -1;
 	}
     }
 #endif
@@ -119,6 +120,7 @@ int xmpi_read_rc(struct xmp_context *ctx)
 	getval_yn(o->flags, "pan", XMP_CTL_DYNPAN);
 	getval_yn(o->flags, "filter", XMP_CTL_FILTER);
 	getval_yn(o->outfmt, "mono", XMP_FMT_MONO);
+	getval_no("amplify", o->amplify);
 	getval_no("mix", o->mix);
 	getval_no("crunch", o->crunch);
 	getval_no("chorus", o->chorus);
@@ -127,12 +129,20 @@ int xmpi_read_rc(struct xmp_context *ctx)
 	getval_no("time", o->time);
 	getval_no("verbosity", o->verbosity);
 
+	/* don't parse clickfilter in general config */
+	/* don't parse vblank in general config */
+
 	if (!strcmp (var, "driver")) {
 	    strncpy (drive_id, val, 31);
 	    o->drv_id = drive_id;
 	    continue;
 	}
 
+	if (!strcmp (var, "amplify")) {
+	    o->amplify = atoi (val);
+	    continue;
+	}
+
 	if (!strcmp (var, "bits")) {
 	    o->resol = atoi (val);
 	    if (o->resol != 16 || o->resol != 8 || o->resol != 0)
@@ -149,11 +159,11 @@ int xmpi_read_rc(struct xmp_context *ctx)
 
     fclose (rc);
 
-    return XMP_OK;
+    return 0;
 }
 
 
-static void parse_modconf(struct xmp_context *ctx, char *fn, unsigned crc, unsigned size)
+static void parse_modconf(struct xmp_context *ctx, char *s, unsigned crc, unsigned size)
 {
     struct xmp_player_context *p = &ctx->p;
     struct xmp_mod_context *m = &p->m;
@@ -162,13 +172,13 @@ static void parse_modconf(struct xmp_context *ctx, char *fn, unsigned crc, unsig
     char *hash, *var, *val, line[256];
     int active = 0;
 
-    if ((rc = fopen (fn, "r")) == NULL)
+    if ((rc = fopen(s, "r")) == NULL)
 	return;
 
-    while (!feof (rc)) {
+    while (!feof(rc)) {
 	memset (line, 0, 256);
-	fscanf (rc, "%255[^\n]", line);
-	fgetc (rc);
+	fscanf(rc, "%255[^\n]", line);
+	fgetc(rc);
 
 	/* Delete comments */
 	if ((hash = strchr (line, '#')))
@@ -181,7 +191,7 @@ static void parse_modconf(struct xmp_context *ctx, char *fn, unsigned crc, unsig
 		active = (strtoul (&line[1], NULL, 0) == crc &&
 	 	    strtoul (val, NULL, 0) == size);
 		if (active && o->verbosity > 2)
-		    report ("Matching CRC in %s (%u)\n", fn, crc);
+		    report("Quirk found in %s\n", s);
 	    }
 	    continue;
  	}
@@ -204,35 +214,48 @@ static void parse_modconf(struct xmp_context *ctx, char *fn, unsigned crc, unsig
 	getval_yn(m->fetch, "filter", XMP_CTL_FILTER);
 	getval_yn(m->fetch, "fixloop", XMP_CTL_FIXLOOP);
 	getval_yn(m->fetch, "fx9bug", XMP_CTL_FX9BUG);
+	getval_yn(m->fetch, "vblank", XMP_CTL_VBLANK);
 	getval_yn(o->outfmt, "mono", XMP_FMT_MONO);
 	getval_no("mix", o->mix);
 	getval_no("crunch", o->crunch);
 	getval_no("chorus", o->chorus);
 	getval_no("reverb", o->reverb);
+
+	if (!strcmp (var, "clickfilter")) {
+	    if (get_yesno(val)) {
+		o->flags |= XMP_CTL_FILTER;
+		o->cf_cutoff = 0xfd;
+	    }
+	    continue;
+	}
     }
 
     fclose (rc);
 }
 
 
-void xmpi_read_modconf(struct xmp_context *ctx, unsigned crc, unsigned size)
+void _xmp_read_modconf(struct xmp_context *ctx, uint32 crc, long size)
 {
 #if defined __EMX__
-    char myrc[MAXPATHLEN];
+    char myrc[PATH_MAX];
     char *home = getenv ("HOME");
 
-    snprintf(myrc, MAXPATHLEN, "%s\\.xmp\\modules.conf", home);
+    snprintf(myrc, PATH_MAX, "%s\\.xmp\\modules.conf", home);
     parse_modconf(ctx, "xmp-modules.conf", crc, size);
     parse_modconf(ctx, myrc, crc, size);
 #elif defined __AMIGA__
     parse_modconf(ctx, "PROGDIR:xmp-modules.conf", crc, size);
+#elif defined WIN32
+    char *home = getenv("SystemRoot");
+
+    snprintf(myrc, PATH_MAX, "%s/modules.ini", home);
+    parse_modconf(ctx, myrc, crc, size);
 #else
-    char myrc[MAXPATHLEN];
+    char myrc[PATH_MAX];
     char *home = getenv ("HOME");
 
-    snprintf(myrc, MAXPATHLEN, "%s/.xmp/modules.conf", home);
-    parse_modconf(ctx, SYSCONFDIR "/xmp-modules.conf", crc, size);
+    snprintf(myrc, PATH_MAX, "%s/.xmp/modules.conf", home);
+    parse_modconf(ctx, SYSCONFDIR "/modules.conf", crc, size);
     parse_modconf(ctx, myrc, crc, size);
 #endif
 }
-
