@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2007 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2009 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See doc/COPYING
@@ -30,7 +30,7 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include "xmpi.h"
+#include "common.h"
 #include "effects.h"
 #include "mixer.h"
 
@@ -40,9 +40,9 @@
 #define MAX_GVL		0x40
 
 
-int xmpi_scan_module(struct xmp_context *ctx)
+int _xmp_scan_module(struct xmp_context *ctx)
 {
-    int prm, gvol_slide, f1, f2, ord, ord2;
+    int parm, gvol_slide, f1, f2, p1, p2, ord, ord2;
     int row, last_row, break_row, cnt_row;
     int gvl, bpm, tempo, base_time, chn;
     int alltmp, clock, clock_rst, medbpm;
@@ -81,7 +81,22 @@ int xmpi_scan_module(struct xmp_context *ctx)
     tempo = (tempo = o->tempo ? o->tempo : m->xxh->tpo) ? tempo : TIME;
     base_time = m->rrate;
 
-    ord2 = ord = -1;
+    /* By erlk ozlr <erlk.ozlr@gmail.com>
+     *
+     * xmp doesn't handle really properly the "start" option (-s for the
+     * command-line) for DeusEx's .umx files. These .umx files contain
+     * several loop "tracks" that never join together. That's how they put
+     * multiple musics on each level with a file per level. Each "track"
+     * starts at the same order in all files. The problem is that xmp starts
+     * decoding the module at order 0 and not at the order specified with
+     * the start option. If we have a module that does "0 -> 2 -> 0 -> ...",
+     * we cannot play order 1, even with the supposed right option.
+     *
+     * was: ord2 = ord = -1;
+     */
+    ord2 = -1;
+    ord = ctx->o.start - 1;
+
     gvol_slide = break_row = cnt_row = alltmp = clock_rst = clock = 0;
 
     while (42) {
@@ -104,7 +119,7 @@ int xmpi_scan_module(struct xmp_context *ctx)
 	    continue;
 	}
 
-	if (tab_cnt[ord][break_row])
+	if (break_row < m->xxp[m->xxo[ord]]->rows && tab_cnt[ord][break_row])
 	    break;
 
 	m->xxo_info[ord].gvl = gvl;
@@ -153,69 +168,107 @@ int xmpi_scan_module(struct xmp_context *ctx)
 	    tab_cnt[ord][row]++;
 
 	    for (chn = 0; chn < m->xxh->chn; chn++) {
+		if (row >= m->xxt[m->xxp[m->xxo[ord]]->info[chn].index]->rows)
+		    continue;
+
 		event = &EVENT(m->xxo[ord], chn, row);
 
 		f1 = event->fxt;
+		p1 = event->fxp;
 		f2 = event->f2t;
+		p2 = event->f2p;
 
 		if (f1 == FX_GLOBALVOL || f2 == FX_GLOBALVOL) {
-		    gvl = (f1 == FX_GLOBALVOL) ? event->fxp : event->f2p;
+		    gvl = (f1 == FX_GLOBALVOL) ? p1 : p2;
 		    gvl = gvl > MAX_GVL ? MAX_GVL : gvl < 0 ? 0 : gvl;
 		}
 
 		if (f1 == FX_G_VOLSLIDE || f2 == FX_G_VOLSLIDE) {
-		    prm = (f1 == FX_G_VOLSLIDE) ? event->fxp : event->f2p;
-		    if (prm)
-			gvol_slide = MSN (prm) - LSN (prm);
-		    gvl += gvol_slide *
-			(tempo - !(m->fetch & XMP_CTL_VSALL));
+		    parm = (f1 == FX_G_VOLSLIDE) ? p1 : p2;
+		    if (parm)
+			gvol_slide = MSN(parm) - LSN(parm);
+		    gvl += gvol_slide * (tempo - !(m->fetch & XMP_CTL_VSALL));
 		}
 
-		if (f1 == FX_TEMPO || f2 == FX_TEMPO) {
-		    prm = (f1 == FX_TEMPO) ? event->fxp : event->f2p;
+		if ((f1 == FX_TEMPO && p1) || (f2 == FX_TEMPO && p2)) {
+		    parm = (f1 == FX_TEMPO) ? p1 : p2;
 		    alltmp += cnt_row * tempo * base_time;
 		    cnt_row = 0;
-		    if (prm) {
-			if (prm <= 0x20)
-			    tempo = prm;
-			else {
+		    if (parm) {
+			if (parm <= 0x20)
+			    tempo = parm;
+			else if (~m->fetch & XMP_CTL_VBLANK) {
 			    if (medbpm)
 				clock += 132 * alltmp / 5 / bpm;
 			    else
 				clock += 100 * alltmp / bpm;
 
 			    alltmp = 0;
-			    bpm = prm;
+			    bpm = parm;
 			}
 		    }
 		}
 
-		if (f1 == FX_S3M_TEMPO || f2 == FX_S3M_TEMPO) {
-		    prm = (f1 == FX_S3M_TEMPO) ? event->fxp : event->f2p;
+		if ((f1 == FX_S3M_TEMPO && p1) || (f2 == FX_S3M_TEMPO && p2)) {
+		    parm = (f1 == FX_S3M_TEMPO) ? p1 : p2;
 		    alltmp += cnt_row * tempo * base_time;
 		    cnt_row  = 0;
-		    tempo = prm;
+		    tempo = parm;
+		}
+
+		if ((f1 == FX_S3M_BPM && p1) || (f2 == FX_S3M_BPM && p2)) {
+		    parm = (f1 == FX_S3M_BPM) ? p1 : p2;
+		    alltmp += cnt_row * tempo * base_time;
+		    cnt_row = 0;
+
+		    if (medbpm)
+			clock += 132 * alltmp / 5 / bpm;
+		    else
+			clock += 100 * alltmp / bpm;
+
+		    alltmp = 0;
+		    bpm = parm;
+		}
+
+		if ((f1 == FX_IT_BPM && p1) || (f2 == FX_IT_BPM && p2)) {
+		    parm = (f1 == FX_IT_BPM) ? p1 : p2;
+		    alltmp += cnt_row * tempo * base_time;
+		    cnt_row = 0;
+		    clock += 100 * alltmp / bpm;
+		    alltmp = 0;
+
+		    if (MSN(parm) == 0) {
+			bpm -= LSN(parm);
+			if (bpm < 0x20)
+			    bpm = 0x20;
+		    } else if (MSN(parm) == 1) {
+			bpm += LSN(parm);
+			if (bpm > 0xff)
+			    bpm = 0xff;
+		    } else {
+			bpm = parm;
+		    }
 		}
 
 		if (f1 == FX_JUMP || f2 == FX_JUMP) {
-		    ord2 = (f1 == FX_JUMP) ? event->fxp : event->f2p;
+		    ord2 = (f1 == FX_JUMP) ? p1 : p2;
 		    last_row = 0;
 		}
 
 		if (f1 == FX_BREAK || f2 == FX_BREAK) {
-		    prm = (f1 == FX_BREAK) ? event->fxp : event->f2p;
-		    break_row = 10 * MSN (prm) + LSN (prm);
+		    parm = (f1 == FX_BREAK) ? p1 : p2;
+		    break_row = 10 * MSN(parm) + LSN(parm);
 		    last_row = 0;
 		}
 
 		if (f1 == FX_EXTENDED || f2 == FX_EXTENDED) {
-		    prm = (f1 == FX_EXTENDED) ? event->fxp : event->f2p;
+		    parm = (f1 == FX_EXTENDED) ? p1 : p2;
 
-		    if ((prm >> 4) == EX_PATT_DELAY)
-			alltmp += (prm & 0x0f) * tempo * base_time;
+		    if ((parm >> 4) == EX_PATT_DELAY)
+			alltmp += (parm & 0x0f) * tempo * base_time;
 
-		    if ((prm >> 4) == EX_PATTERN_LOOP) {
-			if (prm &= 0x0f) {
+		    if ((parm >> 4) == EX_PATTERN_LOOP) {
+			if (parm &= 0x0f) {
 			    if (loop_stk[chn]) {
 				if (--loop_stk[chn])
 				    loop_chn = chn + 1;
@@ -226,7 +279,7 @@ int xmpi_scan_module(struct xmp_context *ctx)
 				}
 			    } else {
 				if (loop_row[chn] <= row) {
-				    loop_stk[chn] = prm;
+				    loop_stk[chn] = parm;
 				    loop_chn = chn + 1;
 				    loop_flg++;
 				}
