@@ -1,7 +1,5 @@
 /*
  * XMP plugin for WinAmp
- *
- * $Id: winamp.c,v 1.36 2007/12/05 23:19:51 cmatsuoka Exp $
  */
 
 #include <windows.h>
@@ -11,7 +9,11 @@
 #include <msacm.h>
 #include <math.h>
 
-#include "xmpi.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "common.h"
 #include "in2.h"
 #include "resource.h"
 
@@ -42,6 +44,7 @@ HANDLE load_mutex;
 
 typedef struct {
 	int mixing_freq;
+	int amplify;
 	int force_mono;
 	int interpolation;
 	int filter;
@@ -85,14 +88,29 @@ In_Module mod = {
 	"XMP Plugin " VERSION,
 	0,			/* hMainWindow */
 	0,			/* hDllInstance */
-	"xm;mod;flt;st;m15;it;s3m;stm;stx;mtm;ice;imf;ptm;mdl;"
-	"ult;liq;psm;amf;mmd0;mmd1;mmd2;mmd3;med;med3;med4;dmf;"
-	"rtm;pt3;tcb;dt;gtk;dtt;mgt;arch;dsym;digi;dbm;emod;okt;"
-	"sfx;far;umx;stim;mtp;ims;669;fnk;funk;amd;rad;hsc;alm;"
+	// Leave out the following more obscure extensions, 
+	// cause overloading GetOpenFileName causes missing extensions
+/*
+	"flt;st;stx;m15;ice;imf;mdl;"
+	"liq;psm;mmd0;mmd1;mmd2;mmd3;med3;med4;dmf;"
+	"rtm;pt3;tcb;dt;gtk;dtt;mgt;arch;dsym;dbm;emod;"
+	"sfx;umx;stim;mtp;ims;fnk;funk;"
 	"ac1d;fchs;fcm;fuzz;kris;ksm;mp;p18a;p10c;pru1;pru2;pha;"
 	"wn;unic;tp3;xann;di;eu;p4x;p40;p41;p60a;np1;np2;np3;zen;"
-	"crb;tdd;gmc;gdm\0"
-	"Module formats\0",	/* file extensions */
+	"crb;tdd;gmc;gdm;"
+*/
+	/* file extensions */
+	"xm;it;s3m;s2m;stm;mdl;imf;mtm;ptm;ult;far;669\0"
+		"XMP PC Modules (xm;it;s3m;s2m;stm;mdl;imf;mtm;ptm;ult;far;669)\0"
+	"mod;med;digi;dbm;okt;sfx\0"
+		"XMP Amiga Modules (mod;med;digi;dbm;okt;sfx)\0"
+	"xmz;mdz;itz;s3z;amf\0"
+		"XMP Compressed Modules (xmz;mdz;itz;s3z;amf)\0"
+	"psm;umx;gmc;di;stim\0"
+		"XMP Game Modules (psm;umx;gmc;di;stim)\0"
+	"amd;rad;hsc\0"
+		"XMP Adlib Modules (amd;rad;hsc)\0"
+		,	
 	1,			/* is_seekable */
 	1,			/* uses output */
 	config,
@@ -131,7 +149,7 @@ static char *get_inifile(char *s)
 	if (GetModuleFileName(GetModuleHandle("in_xmp.dll"), s, MAX_PATH)) {
 		if ((c = strrchr(s, '\\')) != NULL)
 			*++c = 0;
-                strncat(s, "plugin.ini", MAX_PATH);
+                strncat(s, "in_xmp.ini", MAX_PATH);
         }
 
 	return s;
@@ -158,6 +176,9 @@ static void stop()
 		}
 		CloseHandle(decode_thread);
 		decode_thread = INVALID_HANDLE_VALUE;
+	} else {
+		// if stopped before entering play loop, do sanity clean
+		xmp_close_audio(ctx);
 	}
 	mod.outMod->Close();
 	mod.SAVSADeInit();
@@ -172,10 +193,11 @@ static void driver_callback(void *b, int i)
 	int t, todo;
 
 	if (dsp) {
+		char * bb = b;
 		/* Winamp dsp support fixed by Mirko Buffoni */
 		while (i > 0) {
 			todo = (i > MIX_BUFSIZE * 2) ? MIX_BUFSIZE : i;
-			memcpy(mix_buffer, b, todo);
+			memcpy(mix_buffer, bb, todo);
 
 			while (mod.outMod->CanWrite() < todo * 2)
 				Sleep(20);
@@ -188,7 +210,7 @@ static void driver_callback(void *b, int i)
 				opt->freq) * numch * ssize;
 			mod.outMod->Write(mix_buffer, n);
 			i -= todo;
-			b += todo;
+			bb += todo;
 		}
 	} else {
 		while (mod.outMod->CanWrite() < i)
@@ -205,6 +227,7 @@ static BOOL CALLBACK config_dialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM l
 	char inifile[MAX_PATH];
 	HANDLE sld, cmb;
 	char *freq[] = { "44100", "22050", "11025", 0 };
+	char *vamp[] = { "Normal", "x2", "x4", "x8", 0 };
 	int i, fidx;
 	char buffer[20];
 
@@ -234,23 +257,39 @@ static BOOL CALLBACK config_dialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM l
 		SendMessage(sld, TBM_SETPOS, (WPARAM)TRUE,
 					(LPARAM)xmp_cfg.pan_amplitude / 10);
 
+		/* Set frequency ComboBox */
 		cmb = GetDlgItem(hDlg, IDC_MIXING_FREQ);
 		SendMessage(cmb, CB_RESETCONTENT, 0, 0);
 
 		for (i = 0; freq[i]; i++)
-			SendMessage(cmb, CB_ADDSTRING, i, (LPARAM)freq[i]);
+			SendMessage(cmb, CB_ADDSTRING, 0, (LPARAM)freq[i]);
 		snprintf(buffer, 20, "%d", xmp_cfg.mixing_freq);
 		fidx = SendMessage(cmb, CB_FINDSTRINGEXACT, -1, (LPARAM)buffer);
 		if (fidx == CB_ERR)
 			fidx = 0;
 		SendMessage(cmb, CB_SETCURSEL, fidx, 0);
 		xmp_cfg.mixing_freq = atoi(freq[fidx]);
+
+		/* Set volume multiplier ComboBox */
+		cmb = GetDlgItem(hDlg, IDC_AMPLIFICATION_FACT);
+		SendMessage(cmb, CB_RESETCONTENT, 0, 0);
+
+		for (i = 0; vamp[i]; i++)
+			SendMessage(cmb, CB_ADDSTRING, 0, (LPARAM)vamp[i]);
+		snprintf(buffer, 20, "%s", vamp[xmp_cfg.amplify]);
+		fidx = SendMessage(cmb, CB_FINDSTRINGEXACT, -1, (LPARAM)buffer);
+		if (fidx == CB_ERR)
+			fidx = 0;
+		SendMessage(cmb, CB_SETCURSEL, fidx, 0);
+		xmp_cfg.amplify = fidx;
 		break;
 	case WM_COMMAND:
 		switch (GET_WM_COMMAND_ID(wParam, lParam)) {
 		case IDOK:
 			xmp_cfg.mixing_freq = atoi(freq[SendMessage(GetDlgItem(
 				hDlg, IDC_MIXING_FREQ), CB_GETCURSEL, 0, 0)]);
+			xmp_cfg.amplify = SendMessage(GetDlgItem(
+				hDlg, IDC_AMPLIFICATION_FACT), CB_GETCURSEL, 0, 0);
 			xmp_cfg.loop = (IsDlgButtonChecked(hDlg,
 				IDC_LOOP) == BST_CHECKED);
 			xmp_cfg.force_mono = (IsDlgButtonChecked(hDlg,
@@ -264,6 +303,7 @@ static BOOL CALLBACK config_dialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM l
 
 			get_inifile(inifile);
 			CFGWRITESTR(mixing_freq);
+			CFGWRITESTR(amplify);
 			CFGWRITESTR(loop);
 			CFGWRITESTR(force_mono);
 			CFGWRITESTR(interpolation);
@@ -323,6 +363,7 @@ static void init()
 	get_inifile(inifile);
 
 	CFGREADINT(mixing_freq, 44100);
+	CFGREADINT(amplify, 1);
 	CFGREADINT(loop, 0);
 	CFGREADINT(fixloops, 0);
 	CFGREADINT(force_mono, 0);
@@ -396,6 +437,7 @@ static int play_file(char *fn)
 		opt->flags &= ~XMP_CTL_FILTER;
 
 	opt->mix = xmp_cfg.pan_amplitude;
+	opt->amplify = xmp_cfg.amplify;
 
 	numch = opt->outfmt & XMP_FMT_MONO ? 1 : 2;
 
@@ -418,10 +460,22 @@ static int play_file(char *fn)
 	load_mutex = CreateMutex(NULL, TRUE, "load_mutex");
 	lret = xmp_load_module(ctx, fn);
 	ReleaseMutex(load_mutex);
+	if (lret < 0) {
+		stop();
+		playing = 0;
+		return -1;
+	}
 
 	xmp_cfg.time = lret;
 	xmp_get_module_info(ctx, &xmp_cfg.mod_info);
 
+	/* Winamp goes nuts if module has zero length */
+	if (xmp_cfg.mod_info.len == 0) {
+		stop();
+		playing = 0;
+		return -1;
+	}
+
 	stopped = 0;
 	decode_thread = (HANDLE)CreateThread(NULL, 0,
 			(LPTHREAD_START_ROUTINE)play_loop,
@@ -507,8 +561,148 @@ static void setpan(int pan)
 	mod.outMod->SetPan(pan);
 }
 
+static void SetColumn( LV_COLUMN* column, int nCol, LPTSTR str, int width, int fmt )
+{
+	column->mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM;
+	column->fmt = fmt;
+	column->cx = width;
+	column->iSubItem = nCol;
+	column->pszText = str;
+}
+
+static void SetItem( LV_ITEM* item, int nRow, int nCol, LPTSTR str )
+{
+	item->mask = LVIF_TEXT | LVIF_STATE;
+	item->state = 0;
+	item->stateMask = 0;
+
+	item->iItem = nRow;
+	item->iSubItem = nCol;
+	item->pszText = str;
+	item->cchTextMax = 40;
+	item->iImage = 0;
+	item->lParam = 0;
+}
+
+static BOOL CALLBACK info_dialog(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+{
+	char tmpbuf[256];
+	HANDLE lvm;
+	LV_COLUMN column;
+	LV_ITEM item;
+	int i,t;
+	struct xmp_module_info mi;
+	struct xmp_player_context *p = &((struct xmp_context *)ctx)->p;
+
+	switch (uMsg) {
+	case WM_CLOSE:
+		EndDialog(hDlg,TRUE);
+		return 0;
+	case WM_INITDIALOG:
+		if (!playing) {
+			ShowWindow(hDlg, SW_HIDE);
+			PostMessage(hDlg, WM_CLOSE, 0, 0);
+		} else {
+			int empty_names = 1;
+
+			xmp_get_module_info(ctx, &mi);
+			// Set module title
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_TITLE), mi.name);
+			// Set module type
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_TYPE), mi.type);
+			// Calculate estimated time
+		    t = _xmp_scan_module((struct xmp_context *)ctx);
+			snprintf(tmpbuf, 256, "%dm %02ds\n",
+				(t + 500) / 60000, ((t + 500) / 1000) % 60);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_EST), tmpbuf);
+			// Module length
+			snprintf(tmpbuf, 256, "%d patterns", mi.len);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_LENGTH), tmpbuf);
+			// Number of channels
+			snprintf(tmpbuf, 256, "%d", mi.chn);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_CHANNELS), tmpbuf);
+			// Number of stored patterns
+			snprintf(tmpbuf, 256, "%d", mi.pat);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_PATTERNS), tmpbuf);
+			// Number of stored samples
+			snprintf(tmpbuf, 256, "%d", mi.smp);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_SAMPLES), tmpbuf);
+			// Number of stored instruments
+			snprintf(tmpbuf, 256, "%d", mi.ins);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_INSTR), tmpbuf);
+			// BPM (TODO: mi.bpm is unsupported atm)
+			snprintf(tmpbuf, 256, "%d", p->m.xxh->bpm);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_BPM), tmpbuf);
+			// Module tempo (TODO: mi.tpo is unsupported atm)
+			snprintf(tmpbuf, 256, "%d", p->m.xxh->tpo);
+			SetWindowText( GetDlgItem(hDlg, IDC_MODULE_TEMPO), tmpbuf);
+
+			lvm = GetDlgItem(hDlg, IDC_SAMPLES);
+			ListView_DeleteAllItems( lvm );
+			// Prepare column headers
+			SetColumn( &column, 0, "Idx", 30, LVCFMT_CENTER );
+			ListView_InsertColumn( lvm, column.iSubItem, &column );
+			SetColumn( &column, 1, "Name", 195, LVCFMT_LEFT );
+			ListView_InsertColumn( lvm, column.iSubItem, &column );
+			SetColumn( &column, 2, "Length", 45, LVCFMT_RIGHT );
+			ListView_InsertColumn( lvm, column.iSubItem, &column );
+
+			for (i=0; i < mi.ins; i++) {
+				if (p->m.xxih[i].name[0] != 0) { 
+					empty_names = 0; break; 
+				}
+			}
+
+			if (!empty_names) {
+				for (i=0; i < mi.ins; i++) {
+					snprintf(tmpbuf, 256, "%d", i+1);
+					SetItem( &item, i, 0, tmpbuf );
+					ListView_InsertItem( lvm, &item );
+					snprintf(tmpbuf, 256, "%s", p->m.xxih[i].name);
+					SetItem( &item, i, 1, tmpbuf );
+					ListView_SetItem( lvm, &item );
+					snprintf(tmpbuf, 256, "%d", p->m.xxs[i].len);
+					SetItem( &item, i, 2, tmpbuf );
+					ListView_SetItem( lvm, &item );
+				}
+			} else {
+				for (i=0; i < mi.smp; i++) {
+					snprintf(tmpbuf, 256, "%d", i+1);
+					SetItem( &item, i, 0, tmpbuf );
+					ListView_InsertItem( lvm, &item );
+					snprintf(tmpbuf, 256, "%s", p->m.xxs[i].name);
+					SetItem( &item, i, 1, tmpbuf );
+					ListView_SetItem( lvm, &item );
+					snprintf(tmpbuf, 256, "%d", p->m.xxs[i].len);
+					SetItem( &item, i, 2, tmpbuf );
+					ListView_SetItem( lvm, &item );
+				}
+			}
+		}
+		break;
+	case WM_COMMAND:
+		switch (GET_WM_COMMAND_ID(wParam, lParam)) {
+		case IDABOUT:
+			about(hDlg);
+			break;
+		case IDOK:
+			/* fall thru */
+		case IDCANCEL:
+			EndDialog(hDlg,TRUE);
+			break;
+		}
+		break;
+	default:
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static int infoDlg(char *fn, HWND hwnd)
 {
+	DialogBox(mod.hDllInstance, (const char *)IDD_INFODLG, hwnd,
+							info_dialog);
 	return 0;
 }
 
